/* Generated by GOB (v2.0.15)   (do not edit directly) */

#include <glib.h>
#include <glib-object.h>


#include "mn-message-view.h"

#ifndef __MN_COMPACT_MESSAGE_VIEW_H__
#define __MN_COMPACT_MESSAGE_VIEW_H__

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/*
 * Type checking and casting macros
 */
#define MN_TYPE_COMPACT_MESSAGE_VIEW	(mn_compact_message_view_get_type())
#define MN_COMPACT_MESSAGE_VIEW(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), mn_compact_message_view_get_type(), MNCompactMessageView)
#define MN_COMPACT_MESSAGE_VIEW_CONST(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), mn_compact_message_view_get_type(), MNCompactMessageView const)
#define MN_COMPACT_MESSAGE_VIEW_CLASS(klass)	G_TYPE_CHECK_CLASS_CAST((klass), mn_compact_message_view_get_type(), MNCompactMessageViewClass)
#define MN_IS_COMPACT_MESSAGE_VIEW(obj)	G_TYPE_CHECK_INSTANCE_TYPE((obj), mn_compact_message_view_get_type ())

#define MN_COMPACT_MESSAGE_VIEW_GET_CLASS(obj)	G_TYPE_INSTANCE_GET_CLASS((obj), mn_compact_message_view_get_type(), MNCompactMessageViewClass)

/*
 * Main object structure
 */
#ifndef __TYPEDEF_MN_COMPACT_MESSAGE_VIEW__
#define __TYPEDEF_MN_COMPACT_MESSAGE_VIEW__
typedef struct _MNCompactMessageView MNCompactMessageView;
#endif
struct _MNCompactMessageView {
	MNMessageView __parent__;
};

/*
 * Class definition
 */
typedef struct _MNCompactMessageViewClass MNCompactMessageViewClass;
struct _MNCompactMessageViewClass {
	MNMessageViewClass __parent__;
};


/*
 * Public methods
 */
GType	mn_compact_message_view_get_type	(void) G_GNUC_CONST;
#line 77 "src/mn-compact-message-view.gob"
GtkWidget * 	mn_compact_message_view_new	(void);
#line 55 "mn-compact-message-view.h"

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
