/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_WEBMAIL_MAILBOX_PROPERTIES_PRIVATE_H__
#define __MN_WEBMAIL_MAILBOX_PROPERTIES_PRIVATE_H__

#include "mn-webmail-mailbox-properties.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

struct _MNWebmailMailboxPropertiesPrivate {
#line 35 "src/mn-webmail-mailbox-properties.gob"
	GtkWidget * inbox_radio;
#line 36 "src/mn-webmail-mailbox-properties.gob"
	GtkWidget * other_radio;
#line 37 "src/mn-webmail-mailbox-properties.gob"
	GtkWidget * folder_entry;
#line 20 "mn-webmail-mailbox-properties-private.h"
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
