/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_MESSAGE_VIEW_PRIVATE_H__
#define __MN_MESSAGE_VIEW_PRIVATE_H__

#include "mn-message-view.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#line 25 "src/mn-message-view.gob"

#include <time.h>

#line 18 "mn-message-view-private.h"
struct _MNMessageViewPrivate {
#line 48 "src/mn-message-view.gob"
	GSList * past_time_cells;
#line 22 "mn-message-view-private.h"
};
#line 94 "src/mn-message-view.gob"
void 	mn_message_view_append_past_time_cell	(MNMessageView * self, time_t past_time, time_t now);
#line 26 "mn-message-view-private.h"

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
