/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_AUTODETECT_MAILBOX_PROPERTIES_PRIVATE_H__
#define __MN_AUTODETECT_MAILBOX_PROPERTIES_PRIVATE_H__

#include "mn-autodetect-mailbox-properties.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

struct _MNAutodetectMailboxPropertiesPrivate {
#line 41 "src/mn-autodetect-mailbox-properties.gob"
	GtkWidget * location_entry;
#line 42 "src/mn-autodetect-mailbox-properties.gob"
	GtkWidget * chooser;
#line 18 "mn-autodetect-mailbox-properties-private.h"
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
