/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_PROPERTIES_DIALOG_PRIVATE_H__
#define __MN_PROPERTIES_DIALOG_PRIVATE_H__

#include "mn-properties-dialog.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#line 24 "src/mn-properties-dialog.gob"

#include "mn-mailbox.h"

#line 18 "mn-properties-dialog-private.h"
struct _MNPropertiesDialogPrivate {
#line 50 "src/mn-properties-dialog.gob"
	GtkWidget * notebook;
#line 53 "src/mn-properties-dialog.gob"
	GtkWidget * play_sound_hbox;
#line 54 "src/mn-properties-dialog.gob"
	GtkWidget * play_sound_check;
#line 55 "src/mn-properties-dialog.gob"
	GtkWidget * play_sound_file_chooser_button;
#line 56 "src/mn-properties-dialog.gob"
	GtkWidget * scrolled;
#line 57 "src/mn-properties-dialog.gob"
	GtkWidget * list;
#line 58 "src/mn-properties-dialog.gob"
	GtkWidget * selected_label;
#line 59 "src/mn-properties-dialog.gob"
	GtkWidget * add;
#line 60 "src/mn-properties-dialog.gob"
	GtkWidget * remove;
#line 61 "src/mn-properties-dialog.gob"
	GtkWidget * properties;
#line 64 "src/mn-properties-dialog.gob"
	GtkWidget * icon_action_mail_reader_radio;
#line 65 "src/mn-properties-dialog.gob"
	GtkWidget * icon_action_open_latest_message_radio;
#line 66 "src/mn-properties-dialog.gob"
	GtkWidget * icon_action_consider_new_mail_as_read_radio;
#line 67 "src/mn-properties-dialog.gob"
	GtkWidget * icon_action_update_radio;
#line 70 "src/mn-properties-dialog.gob"
	GtkWidget * popups_enable_check;
#line 71 "src/mn-properties-dialog.gob"
	GtkWidget * popups_position_section_label;
#line 72 "src/mn-properties-dialog.gob"
	GtkWidget * popups_position_attached_radio;
#line 73 "src/mn-properties-dialog.gob"
	GtkWidget * popups_position_free_radio;
#line 74 "src/mn-properties-dialog.gob"
	GtkWidget * popups_expiration_section_label;
#line 75 "src/mn-properties-dialog.gob"
	GtkWidget * popups_expiration_default_radio;
#line 76 "src/mn-properties-dialog.gob"
	GtkWidget * popups_expiration_never_radio;
#line 77 "src/mn-properties-dialog.gob"
	GtkWidget * popups_expiration_after_radio;
#line 78 "src/mn-properties-dialog.gob"
	GtkWidget * popups_expiration_scale;
#line 80 "src/mn-properties-dialog.gob"
	MNMailbox * test_mailbox;
#line 68 "mn-properties-dialog-private.h"
};
#line 279 "src/mn-properties-dialog.gob"
void 	mn_properties_dialog_toggled_h	(MNPropertiesDialog * self, GtkToggleButton * button);
#line 72 "mn-properties-dialog-private.h"
#line 285 "src/mn-properties-dialog.gob"
void 	mn_properties_dialog_add_clicked_h	(MNPropertiesDialog * self, GtkButton * button);
#line 75 "mn-properties-dialog-private.h"
#line 291 "src/mn-properties-dialog.gob"
void 	mn_properties_dialog_remove_clicked_h	(MNPropertiesDialog * self, GtkButton * button);
#line 78 "mn-properties-dialog-private.h"
#line 297 "src/mn-properties-dialog.gob"
void 	mn_properties_dialog_properties_clicked_h	(MNPropertiesDialog * self, GtkButton * button);
#line 81 "mn-properties-dialog-private.h"
#line 303 "src/mn-properties-dialog.gob"
char * 	mn_properties_dialog_popups_expiration_format_value_h	(MNPropertiesDialog * self, double arg, GtkScale * scale);
#line 84 "mn-properties-dialog-private.h"
#line 309 "src/mn-properties-dialog.gob"
void 	mn_properties_dialog_test_messages_toggled_h	(MNPropertiesDialog * self, GtkToggleButton * button);
#line 87 "mn-properties-dialog-private.h"

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
