/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_POPUPS_PRIVATE_H__
#define __MN_POPUPS_PRIVATE_H__

#include "mn-popups.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

struct _MNPopupsPrivate {
#line 38 "src/mn-popups.gob"
	GHashTable * popups;
#line 16 "mn-popups-private.h"
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
