/* Generated by GOB (v2.0.15)   (do not edit directly) */

#include <glib.h>
#include <glib-object.h>
#ifndef __MN_POPUPS_H__
#define __MN_POPUPS_H__

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/*
 * Type checking and casting macros
 */
#define MN_TYPE_POPUPS	(mn_popups_get_type())
#define MN_POPUPS(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), mn_popups_get_type(), MNPopups)
#define MN_POPUPS_CONST(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), mn_popups_get_type(), MNPopups const)
#define MN_POPUPS_CLASS(klass)	G_TYPE_CHECK_CLASS_CAST((klass), mn_popups_get_type(), MNPopupsClass)
#define MN_IS_POPUPS(obj)	G_TYPE_CHECK_INSTANCE_TYPE((obj), mn_popups_get_type ())

#define MN_POPUPS_GET_CLASS(obj)	G_TYPE_INSTANCE_GET_CLASS((obj), mn_popups_get_type(), MNPopupsClass)

/* Private structure type */
typedef struct _MNPopupsPrivate MNPopupsPrivate;

/*
 * Main object structure
 */
#ifndef __TYPEDEF_MN_POPUPS__
#define __TYPEDEF_MN_POPUPS__
typedef struct _MNPopups MNPopups;
#endif
struct _MNPopups {
	GObject __parent__;
	/*< private >*/
	MNPopupsPrivate *_priv;
};

/*
 * Class definition
 */
typedef struct _MNPopupsClass MNPopupsClass;
struct _MNPopupsClass {
	GObjectClass __parent__;
};


/*
 * Public methods
 */
GType	mn_popups_get_type	(void) G_GNUC_CONST;
#line 265 "src/mn-popups.gob"
MNPopups * 	mn_popups_new	(void);
#line 56 "mn-popups.h"

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
