/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_MAIL_ICON_WIDGET_PRIVATE_H__
#define __MN_MAIL_ICON_WIDGET_PRIVATE_H__

#include "mn-mail-icon-widget.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

struct _MNMailIconWidgetPrivate {
#line 40 "src/mn-mail-icon-widget.gob"
	gboolean blinking;
#line 67 "src/mn-mail-icon-widget.gob"
	PangoLayout * count_layout;
#line 69 "src/mn-mail-icon-widget.gob"
	int count;
#line 86 "src/mn-mail-icon-widget.gob"
	unsigned int blink_timeout_id;
#line 87 "src/mn-mail-icon-widget.gob"
	gboolean is_on;
#line 89 "src/mn-mail-icon-widget.gob"
	char * stock_id;
#line 90 "src/mn-mail-icon-widget.gob"
	GtkIconSize icon_size;
#line 28 "mn-mail-icon-widget-private.h"
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
