/* Generated by GOB (v2.0.15)   (do not edit directly) */

#include <glib.h>
#include <glib-object.h>


#include "mn-pi-mailbox.h"

#ifndef __MN_POP3_MAILBOX_H__
#define __MN_POP3_MAILBOX_H__

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */



extern int mn_pop3_mailbox_default_ports[MN_PI_MAILBOX_N_CONNECTION_TYPES];


/*
 * Type checking and casting macros
 */
#define MN_TYPE_POP3_MAILBOX	(mn_pop3_mailbox_get_type())
#define MN_POP3_MAILBOX(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), mn_pop3_mailbox_get_type(), MNPOP3Mailbox)
#define MN_POP3_MAILBOX_CONST(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), mn_pop3_mailbox_get_type(), MNPOP3Mailbox const)
#define MN_POP3_MAILBOX_CLASS(klass)	G_TYPE_CHECK_CLASS_CAST((klass), mn_pop3_mailbox_get_type(), MNPOP3MailboxClass)
#define MN_IS_POP3_MAILBOX(obj)	G_TYPE_CHECK_INSTANCE_TYPE((obj), mn_pop3_mailbox_get_type ())

#define MN_POP3_MAILBOX_GET_CLASS(obj)	G_TYPE_INSTANCE_GET_CLASS((obj), mn_pop3_mailbox_get_type(), MNPOP3MailboxClass)

/* Private structure type */
typedef struct _MNPOP3MailboxPrivate MNPOP3MailboxPrivate;

/*
 * Main object structure
 */
#ifndef __TYPEDEF_MN_POP3_MAILBOX__
#define __TYPEDEF_MN_POP3_MAILBOX__
typedef struct _MNPOP3Mailbox MNPOP3Mailbox;
#endif
struct _MNPOP3Mailbox {
	MNPIMailbox __parent__;
	/*< private >*/
	MNPOP3MailboxPrivate *_priv;
};

/*
 * Class definition
 */
typedef struct _MNPOP3MailboxClass MNPOP3MailboxClass;
struct _MNPOP3MailboxClass {
	MNPIMailboxClass __parent__;
};


/*
 * Public methods
 */
GType	mn_pop3_mailbox_get_type	(void) G_GNUC_CONST;
#line 1292 "src/mn-pop3-mailbox.gob"
char * 	mn_pop3_mailbox_build_name	(const char * username,
					const char * server);
#line 65 "mn-pop3-mailbox.h"

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
