/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_VFS_MAILBOX_PRIVATE_H__
#define __MN_VFS_MAILBOX_PRIVATE_H__

#include "mn-vfs-mailbox.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#line 30 "src/mn-vfs-mailbox.gob"

#include "mn-vfs-mailbox-backend.h"

#line 18 "mn-vfs-mailbox-private.h"
struct _MNVFSMailboxPrivate {
#line 80 "src/mn-vfs-mailbox.gob"
	GSList * monitors;
#line 81 "src/mn-vfs-mailbox.gob"
	MNVFSMailboxBackend * backend;
#line 82 "src/mn-vfs-mailbox.gob"
	unsigned int update_poll_id;
#line 84 "src/mn-vfs-mailbox.gob"
	GMutex * mutex;
#line 28 "mn-vfs-mailbox-private.h"
};
#line 201 "src/mn-vfs-mailbox.gob"
void 	mn_vfs_mailbox_monitor	(MNVFSMailbox * self, const char * uri, MNVFSMailboxBackend * owner, GnomeVFSMonitorType type, GnomeVFSMonitorCallback callback, gpointer user_data);
#line 32 "mn-vfs-mailbox-private.h"
#line 294 "src/mn-vfs-mailbox.gob"
void 	mn_vfs_mailbox_remove_monitors_by_owner	(MNVFSMailbox * self, MNVFSMailboxBackend * owner);
#line 35 "mn-vfs-mailbox-private.h"

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
