/* Generated by GOB (v2.0.15)   (do not edit directly) */

#include <glib.h>
#include <glib-object.h>


#include <gtk/gtk.h>
#include "mn-mailbox-properties.h"

#ifndef __MN_AUTHENTICATED_MAILBOX_PROPERTIES_H__
#define __MN_AUTHENTICATED_MAILBOX_PROPERTIES_H__

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/*
 * Type checking and casting macros
 */
#define MN_TYPE_AUTHENTICATED_MAILBOX_PROPERTIES	(mn_authenticated_mailbox_properties_get_type())
#define MN_AUTHENTICATED_MAILBOX_PROPERTIES(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), mn_authenticated_mailbox_properties_get_type(), MNAuthenticatedMailboxProperties)
#define MN_AUTHENTICATED_MAILBOX_PROPERTIES_CONST(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), mn_authenticated_mailbox_properties_get_type(), MNAuthenticatedMailboxProperties const)
#define MN_AUTHENTICATED_MAILBOX_PROPERTIES_CLASS(klass)	G_TYPE_CHECK_CLASS_CAST((klass), mn_authenticated_mailbox_properties_get_type(), MNAuthenticatedMailboxPropertiesClass)
#define MN_IS_AUTHENTICATED_MAILBOX_PROPERTIES(obj)	G_TYPE_CHECK_INSTANCE_TYPE((obj), mn_authenticated_mailbox_properties_get_type ())

#define MN_AUTHENTICATED_MAILBOX_PROPERTIES_GET_CLASS(obj)	G_TYPE_INSTANCE_GET_CLASS((obj), mn_authenticated_mailbox_properties_get_type(), MNAuthenticatedMailboxPropertiesClass)

/* Private structure type */
typedef struct _MNAuthenticatedMailboxPropertiesPrivate MNAuthenticatedMailboxPropertiesPrivate;

/*
 * Main object structure
 */
#ifndef __TYPEDEF_MN_AUTHENTICATED_MAILBOX_PROPERTIES__
#define __TYPEDEF_MN_AUTHENTICATED_MAILBOX_PROPERTIES__
typedef struct _MNAuthenticatedMailboxProperties MNAuthenticatedMailboxProperties;
#endif
struct _MNAuthenticatedMailboxProperties {
	MNMailboxProperties __parent__;
	/*< private >*/
	GtkWidget * account_vbox; /* protected */
	GtkWidget * username_vbox; /* protected */
	GtkWidget * username_label; /* protected */
	GtkWidget * username_entry; /* protected */
	GtkWidget * password_vbox; /* protected */
	GtkWidget * password_label; /* protected */
	GtkWidget * password_entry; /* protected */
	MNAuthenticatedMailboxPropertiesPrivate *_priv;
};

/*
 * Class definition
 */
typedef struct _MNAuthenticatedMailboxPropertiesClass MNAuthenticatedMailboxPropertiesClass;
struct _MNAuthenticatedMailboxPropertiesClass {
	MNMailboxPropertiesClass __parent__;
};


/*
 * Public methods
 */
GType	mn_authenticated_mailbox_properties_get_type	(void) G_GNUC_CONST;

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
