/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_PI_MAILBOX_PRIVATE_H__
#define __MN_PI_MAILBOX_PRIVATE_H__

#include "mn-pi-mailbox.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#line 28 "src/mn-pi-mailbox.gob"

#include <gtk/gtk.h>
#include "mn-client-session.h"

#define MN_PI_MAILBOX_SESSION_PRIVATE \
  MNMailbox			*mailbox;	\
  MNAuthenticatedMailbox	*auth_mailbox;	\
  MNPIMailbox			*pi_mailbox;	\
  MNClientSession		*session

#line 25 "mn-pi-mailbox-private.h"
struct _MNPIMailboxPrivate {
#line 85 "src/mn-pi-mailbox.gob"
	GtkWidget * untrusted_dialog;
#line 29 "mn-pi-mailbox-private.h"
};
#line 156 "src/mn-pi-mailbox.gob"
gboolean 	mn_pi_mailbox_split_uri	(const char * uri, int maxlen, char * scheme, char * auth, char * location);
#line 33 "mn-pi-mailbox-private.h"
#line 173 "src/mn-pi-mailbox.gob"
gboolean 	mn_pi_mailbox_split_uri_auth	(const char * auth, int maxlen, char * username, char * authmech, gboolean * has_authmech);
#line 36 "mn-pi-mailbox-private.h"
#line 195 "src/mn-pi-mailbox.gob"
void 	mn_pi_mailbox_split_uri_hostport	(const char * hostport, int maxlen, char * hostname, int * port);
#line 39 "mn-pi-mailbox-private.h"
#line 223 "src/mn-pi-mailbox.gob"
void 	mn_pi_mailbox_session_private_init	(MNPIMailbox * self, MNClientSessionPrivate * priv);
#line 42 "mn-pi-mailbox-private.h"
#line 231 "src/mn-pi-mailbox.gob"
gboolean 	mn_pi_mailbox_sasl_get_credentials_cb	(MNClientSession * session, MNClientSessionPrivate * priv, const char ** username, const char ** password);
#line 45 "mn-pi-mailbox-private.h"
#line 264 "src/mn-pi-mailbox.gob"
gboolean 	mn_pi_mailbox_ssl_trust_server_cb	(MNClientSession * session, const char * server, int port, const char * fingerprint, const char * verify_error, MNClientSessionPrivate * priv);
#line 48 "mn-pi-mailbox-private.h"
#line 384 "src/mn-pi-mailbox.gob"
void 	mn_pi_mailbox_notice_cb	(MNClientSession * session, const char * message, MNClientSessionPrivate * priv);
#line 51 "mn-pi-mailbox-private.h"
#line 392 "src/mn-pi-mailbox.gob"
void 	mn_pi_mailbox_warning_cb	(MNClientSession * session, const char * message, MNClientSessionPrivate * priv);
#line 54 "mn-pi-mailbox-private.h"

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
