/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_GMIME_STREAM_VFS_PRIVATE_H__
#define __MN_GMIME_STREAM_VFS_PRIVATE_H__

#include "mn-gmime-stream-vfs.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

struct _MNGMimeStreamVFSPrivate {
#line 61 "src/mn-gmime-stream-vfs.gob"
	GnomeVFSHandle * handle;
#line 62 "src/mn-gmime-stream-vfs.gob"
	gboolean handle_owned;
#line 64 "src/mn-gmime-stream-vfs.gob"
	char * uri;
#line 65 "src/mn-gmime-stream-vfs.gob"
	gboolean eof;
#line 22 "mn-gmime-stream-vfs-private.h"
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
