/* Generated by GOB (v2.0.15)   (do not edit directly) */

/* End world hunger, donate to the World Food Programme, http://www.wfp.org */

#define GOB_VERSION_MAJOR 2
#define GOB_VERSION_MINOR 0
#define GOB_VERSION_PATCHLEVEL 15

#define selfp (self->_priv)

#include <string.h> /* memset() */

#include "mn-properties-dialog.h"

#include "mn-properties-dialog-private.h"

#ifdef G_LIKELY
#define ___GOB_LIKELY(expr) G_LIKELY(expr)
#define ___GOB_UNLIKELY(expr) G_UNLIKELY(expr)
#else /* ! G_LIKELY */
#define ___GOB_LIKELY(expr) (expr)
#define ___GOB_UNLIKELY(expr) (expr)
#endif /* G_LIKELY */

#line 28 "src/mn-properties-dialog.gob"

#include <glib/gi18n.h>
#include <gtk/gtk.h>
#include "mn-conf.h"
#include "mn-util.h"
#include "mn-mailbox-view.h"
#include "mn-shell.h"
#include "mn-file-chooser-button.h"
#include "mn-sound-file-chooser-dialog.h"
#include "mn-test-mailbox.h"
#include "mn-non-linear-range.h"

static const MNNonLinearRangeBlock expiration_blocks[] = {
  { MN_SECS(1),		MN_SECS(10),	MN_SECS(1)	},
  { MN_SECS(15),	MN_SECS(60),	MN_SECS(5)	},
  { MN_MINS(2),		MN_MINS(10),	MN_MINS(1)	},
  { MN_MINS(15),	MN_MINS(60),	MN_MINS(5)	}
};

#line 46 "mn-properties-dialog.c"
/* self casting macros */
#define SELF(x) MN_PROPERTIES_DIALOG(x)
#define SELF_CONST(x) MN_PROPERTIES_DIALOG_CONST(x)
#define IS_SELF(x) MN_IS_PROPERTIES_DIALOG(x)
#define TYPE_SELF MN_TYPE_PROPERTIES_DIALOG
#define SELF_CLASS(x) MN_PROPERTIES_DIALOG_CLASS(x)

#define SELF_GET_CLASS(x) MN_PROPERTIES_DIALOG_GET_CLASS(x)

/* self typedefs */
typedef MNPropertiesDialog Self;
typedef MNPropertiesDialogClass SelfClass;

/* here are local prototypes */
#line 0 "src/mn-properties-dialog.gob"
static void mn_properties_dialog_class_init (MNPropertiesDialogClass * c);
#line 63 "mn-properties-dialog.c"
#line 82 "src/mn-properties-dialog.gob"
static void mn_properties_dialog_init (MNPropertiesDialog * self);
#line 66 "mn-properties-dialog.c"
#line 178 "src/mn-properties-dialog.gob"
static GtkFileChooserDialog * mn_properties_dialog_play_sound_file_chooser_create_dialog_cb (gpointer data);
#line 69 "mn-properties-dialog.c"
#line 184 "src/mn-properties-dialog.gob"
static void mn_properties_dialog_remove_test_mailbox (MNPropertiesDialog * self);
#line 72 "mn-properties-dialog.c"
#line 195 "src/mn-properties-dialog.gob"
static void mn_properties_dialog_response_h (MNPropertiesDialog * self, int response, gpointer user_data);
#line 75 "mn-properties-dialog.c"
#line 222 "src/mn-properties-dialog.gob"
static void mn_properties_dialog_update_selected_label (MNPropertiesDialog * self);
#line 78 "mn-properties-dialog.c"
#line 245 "src/mn-properties-dialog.gob"
static void mn_properties_dialog_update_sensitivity (MNPropertiesDialog * self);
#line 81 "mn-properties-dialog.c"

/* pointer to the class of our parent */
static MNDialogClass *parent_class = NULL;

/* Short form macros */
#define self_play_sound_file_chooser_create_dialog_cb mn_properties_dialog_play_sound_file_chooser_create_dialog_cb
#define self_remove_test_mailbox mn_properties_dialog_remove_test_mailbox
#define self_response_h mn_properties_dialog_response_h
#define self_update_selected_label mn_properties_dialog_update_selected_label
#define self_update_sensitivity mn_properties_dialog_update_sensitivity
#define self_toggled_h mn_properties_dialog_toggled_h
#define self_add_clicked_h mn_properties_dialog_add_clicked_h
#define self_remove_clicked_h mn_properties_dialog_remove_clicked_h
#define self_properties_clicked_h mn_properties_dialog_properties_clicked_h
#define self_popups_expiration_format_value_h mn_properties_dialog_popups_expiration_format_value_h
#define self_test_messages_toggled_h mn_properties_dialog_test_messages_toggled_h
GType
mn_properties_dialog_get_type (void)
{
	static GType type = 0;

	if ___GOB_UNLIKELY(type == 0) {
		static const GTypeInfo info = {
			sizeof (MNPropertiesDialogClass),
			(GBaseInitFunc) NULL,
			(GBaseFinalizeFunc) NULL,
			(GClassInitFunc) mn_properties_dialog_class_init,
			(GClassFinalizeFunc) NULL,
			NULL /* class_data */,
			sizeof (MNPropertiesDialog),
			0 /* n_preallocs */,
			(GInstanceInitFunc) mn_properties_dialog_init,
			NULL
		};

		type = g_type_register_static (MN_TYPE_DIALOG, "MNPropertiesDialog", &info, (GTypeFlags)0);
	}

	return type;
}

/* a macro for creating a new object of our type */
#define GET_NEW ((MNPropertiesDialog *)g_object_new(mn_properties_dialog_get_type(), NULL))

/* a function for creating a new object of our type */
#include <stdarg.h>
static MNPropertiesDialog * GET_NEW_VARG (const char *first, ...) G_GNUC_UNUSED;
static MNPropertiesDialog *
GET_NEW_VARG (const char *first, ...)
{
	MNPropertiesDialog *ret;
	va_list ap;
	va_start (ap, first);
	ret = (MNPropertiesDialog *)g_object_new_valist (mn_properties_dialog_get_type (), first, ap);
	va_end (ap);
	return ret;
}


static void
___finalize(GObject *obj_self)
{
#define __GOB_FUNCTION__ "MN:Properties:Dialog::finalize"
	MNPropertiesDialog *self G_GNUC_UNUSED = MN_PROPERTIES_DIALOG (obj_self);
	gpointer priv G_GNUC_UNUSED = self->_priv;
	if(G_OBJECT_CLASS(parent_class)->finalize) \
		(* G_OBJECT_CLASS(parent_class)->finalize)(obj_self);
#define test_mailbox (self->_priv->test_mailbox)
#define VAR test_mailbox
	{
#line 80 "src/mn-properties-dialog.gob"
	 self_remove_test_mailbox(self); }
#line 154 "mn-properties-dialog.c"
	memset(&test_mailbox, 0, sizeof(test_mailbox));
#undef VAR
#undef test_mailbox
}
#undef __GOB_FUNCTION__

static void 
mn_properties_dialog_class_init (MNPropertiesDialogClass * c G_GNUC_UNUSED)
{
#define __GOB_FUNCTION__ "MN:Properties:Dialog::class_init"
	GObjectClass *g_object_class G_GNUC_UNUSED = (GObjectClass*) c;

	g_type_class_add_private(c,sizeof(MNPropertiesDialogPrivate));

	parent_class = g_type_class_ref (MN_TYPE_DIALOG);

	g_object_class->finalize = ___finalize;
}
#undef __GOB_FUNCTION__
#line 82 "src/mn-properties-dialog.gob"
static void 
mn_properties_dialog_init (MNPropertiesDialog * self G_GNUC_UNUSED)
{
#line 178 "mn-properties-dialog.c"
#define __GOB_FUNCTION__ "MN:Properties:Dialog::init"
	self->_priv = G_TYPE_INSTANCE_GET_PRIVATE(self,MN_TYPE_PROPERTIES_DIALOG,MNPropertiesDialogPrivate);
 {
#line 83 "src/mn-properties-dialog.gob"

    GtkTreeSelection *selection;

    mn_container_create_interface(GTK_CONTAINER(self),
				  PKGDATADIR G_DIR_SEPARATOR_S "properties-dialog.glade",
				  "main_vbox",
				  "mn_properties_dialog_",
				  "notebook", &selfp->notebook,
				  "play_sound_hbox", &selfp->play_sound_hbox,
				  "play_sound_check", &selfp->play_sound_check,
				  "scrolled", &selfp->scrolled,
				  "selected_label", &selfp->selected_label,
				  "add", &selfp->add,
				  "remove", &selfp->remove,
				  "properties", &selfp->properties,
				  "icon_action_mail_reader_radio", &selfp->icon_action_mail_reader_radio,
				  "icon_action_open_latest_message_radio", &selfp->icon_action_open_latest_message_radio,
				  "icon_action_consider_new_mail_as_read_radio", &selfp->icon_action_consider_new_mail_as_read_radio,
				  "icon_action_update_radio", &selfp->icon_action_update_radio,
				  "popups_enable_check", &selfp->popups_enable_check,
				  "popups_position_section_label", &selfp->popups_position_section_label,
				  "popups_position_attached_radio", &selfp->popups_position_attached_radio,
				  "popups_position_free_radio", &selfp->popups_position_free_radio,
				  "popups_expiration_section_label", &selfp->popups_expiration_section_label,
				  "popups_expiration_default_radio", &selfp->popups_expiration_default_radio,
				  "popups_expiration_never_radio", &selfp->popups_expiration_never_radio,
				  "popups_expiration_after_radio", &selfp->popups_expiration_after_radio,
				  "popups_expiration_scale", &selfp->popups_expiration_scale,
				  NULL);

    /* translators: header capitalization */
    gtk_window_set_title(GTK_WINDOW(self), _("Mail Notification Properties"));

    gtk_dialog_add_buttons(GTK_DIALOG(self),
			   GTK_STOCK_HELP, GTK_RESPONSE_HELP,
			   GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
			   NULL);

    selfp->play_sound_file_chooser_button = mn_file_chooser_button_new(self_play_sound_file_chooser_create_dialog_cb, NULL);
    gtk_box_pack_start(GTK_BOX(selfp->play_sound_hbox), selfp->play_sound_file_chooser_button, TRUE, TRUE, 0);
    gtk_widget_show(selfp->play_sound_file_chooser_button);

    selfp->list = mn_mailbox_view_new();
    gtk_container_add(GTK_CONTAINER(selfp->scrolled), selfp->list);
    gtk_widget_show(selfp->list);

    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(selfp->list));
    g_object_connect(selection,
		     "swapped-signal::changed", self_update_selected_label, self,
		     "swapped-signal::changed", self_update_sensitivity, self,
		     NULL);

    mn_setup_dnd(selfp->scrolled);

    mn_non_linear_range_setup_static(GTK_RANGE(selfp->popups_expiration_scale), expiration_blocks, G_N_ELEMENTS(expiration_blocks));

    mn_conf_link_window(GTK_WINDOW(self), MN_CONF_PROPERTIES_DIALOG);
    mn_conf_link_object(selfp->play_sound_check, MN_CONF_SOUNDS_NEW_MAIL_ENABLED, "active");
    mn_conf_link_object(selfp->play_sound_file_chooser_button, MN_CONF_SOUNDS_NEW_MAIL_FILE, "filename");
    mn_conf_link_object(selfp->popups_enable_check, MN_CONF_POPUPS_ENABLED, "active");
    mn_conf_link_non_linear_range(GTK_RANGE(selfp->popups_expiration_scale), MN_CONF_POPUPS_EXPIRATION_DELAY);
    mn_conf_link_radio_group_to_enum(MN_TYPE_ACTION,
				     MN_CONF_CLICK_ACTION,
				     selfp->icon_action_mail_reader_radio, MN_ACTION_LAUNCH_MAIL_READER,
				     selfp->icon_action_open_latest_message_radio, MN_ACTION_OPEN_LATEST_MESSAGE,
				     selfp->icon_action_consider_new_mail_as_read_radio, MN_ACTION_CONSIDER_NEW_MAIL_AS_READ,
				     selfp->icon_action_update_radio, MN_ACTION_UPDATE_MAIL_STATUS,
				     NULL);
    mn_conf_link_radio_group_to_enum(MN_TYPE_POPUP_POSITION,
				     MN_CONF_POPUPS_POSITION,
				     selfp->popups_position_attached_radio, MN_POPUP_POSITION_ATTACHED,
				     selfp->popups_position_free_radio, MN_POPUP_POSITION_FREE,
				     NULL);
    mn_conf_link_radio_group_to_enum(MN_TYPE_EXPIRATION_ENABLED,
				     MN_CONF_POPUPS_EXPIRATION_ENABLED,
				     selfp->popups_expiration_default_radio, MN_EXPIRATION_ENABLED_DEFAULT,
				     selfp->popups_expiration_never_radio, MN_EXPIRATION_ENABLED_FALSE,
				     selfp->popups_expiration_after_radio, MN_EXPIRATION_ENABLED_TRUE,
				     NULL);

    self_update_selected_label(self);
    self_update_sensitivity(self);

    /*
     * HIG chapter 3:
     * "When opening a dialog, provide initial keyboard focus to the
     * component that you expect users to operate first. This focus is
     * especially important for users who must use a keyboard to
     * navigate your application."
     */
    gtk_widget_grab_focus(selfp->add);

    g_signal_connect(self, "response", G_CALLBACK(self_response_h), NULL);
  
#line 277 "mn-properties-dialog.c"
 }
}
#undef __GOB_FUNCTION__



#line 178 "src/mn-properties-dialog.gob"
static GtkFileChooserDialog * 
mn_properties_dialog_play_sound_file_chooser_create_dialog_cb (gpointer data)
{
#line 288 "mn-properties-dialog.c"
#define __GOB_FUNCTION__ "MN:Properties:Dialog::play_sound_file_chooser_create_dialog_cb"
{
#line 180 "src/mn-properties-dialog.gob"
	
    return GTK_FILE_CHOOSER_DIALOG(mn_sound_file_chooser_dialog_new());
  }}
#line 295 "mn-properties-dialog.c"
#undef __GOB_FUNCTION__

#line 184 "src/mn-properties-dialog.gob"
static void 
mn_properties_dialog_remove_test_mailbox (MNPropertiesDialog * self)
{
#line 302 "mn-properties-dialog.c"
#define __GOB_FUNCTION__ "MN:Properties:Dialog::remove_test_mailbox"
#line 184 "src/mn-properties-dialog.gob"
	g_return_if_fail (self != NULL);
#line 184 "src/mn-properties-dialog.gob"
	g_return_if_fail (MN_IS_PROPERTIES_DIALOG (self));
#line 308 "mn-properties-dialog.c"
{
#line 186 "src/mn-properties-dialog.gob"
	
    if (selfp->test_mailbox)
      {
	mn_mailboxes_remove(mn_shell->mailboxes, selfp->test_mailbox);
	g_object_unref(selfp->test_mailbox);
	selfp->test_mailbox = NULL;
      }
  }}
#line 319 "mn-properties-dialog.c"
#undef __GOB_FUNCTION__

#line 195 "src/mn-properties-dialog.gob"
static void 
mn_properties_dialog_response_h (MNPropertiesDialog * self, int response, gpointer user_data)
{
#line 326 "mn-properties-dialog.c"
#define __GOB_FUNCTION__ "MN:Properties:Dialog::response_h"
#line 195 "src/mn-properties-dialog.gob"
	g_return_if_fail (self != NULL);
#line 195 "src/mn-properties-dialog.gob"
	g_return_if_fail (MN_IS_PROPERTIES_DIALOG (self));
#line 332 "mn-properties-dialog.c"
{
#line 197 "src/mn-properties-dialog.gob"
	
    switch (response)
      {
      case GTK_RESPONSE_HELP:
	{
	  int current_page;
	  static const char *sections[] = {
	    "properties-general",
	    "properties-status-icon",
	    "properties-message-popups"
	  };

	  current_page = gtk_notebook_get_current_page(GTK_NOTEBOOK(selfp->notebook));
	  g_assert(current_page >= 0 && current_page < G_N_ELEMENTS(sections));

	  mn_show_help(GTK_WINDOW(self), sections[current_page]);
	  break;
	}

      case GTK_RESPONSE_CLOSE:
	gtk_widget_destroy(GTK_WIDGET(self));
	break;
      }
  }}
#line 359 "mn-properties-dialog.c"
#undef __GOB_FUNCTION__

#line 222 "src/mn-properties-dialog.gob"
static void 
mn_properties_dialog_update_selected_label (MNPropertiesDialog * self)
{
#line 366 "mn-properties-dialog.c"
#define __GOB_FUNCTION__ "MN:Properties:Dialog::update_selected_label"
#line 222 "src/mn-properties-dialog.gob"
	g_return_if_fail (self != NULL);
#line 222 "src/mn-properties-dialog.gob"
	g_return_if_fail (MN_IS_PROPERTIES_DIALOG (self));
#line 372 "mn-properties-dialog.c"
{
#line 224 "src/mn-properties-dialog.gob"
	
    GtkTreeSelection *selection;
    int n_rows;

    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(selfp->list));
    n_rows = gtk_tree_selection_count_selected_rows(selection);

    if (n_rows == 0)
      gtk_label_set_text(GTK_LABEL(selfp->selected_label), _("No mailbox selected."));
    else
      {
	char *str;

	str = g_strdup_printf(ngettext("%i mailbox selected.",
				       "%i mailboxes selected.",
				       n_rows), n_rows);
	gtk_label_set_text(GTK_LABEL(selfp->selected_label), str);
	g_free(str);
      }
  }}
#line 395 "mn-properties-dialog.c"
#undef __GOB_FUNCTION__

#line 245 "src/mn-properties-dialog.gob"
static void 
mn_properties_dialog_update_sensitivity (MNPropertiesDialog * self)
{
#line 402 "mn-properties-dialog.c"
#define __GOB_FUNCTION__ "MN:Properties:Dialog::update_sensitivity"
#line 245 "src/mn-properties-dialog.gob"
	g_return_if_fail (self != NULL);
#line 245 "src/mn-properties-dialog.gob"
	g_return_if_fail (MN_IS_PROPERTIES_DIALOG (self));
#line 408 "mn-properties-dialog.c"
{
#line 247 "src/mn-properties-dialog.gob"
	
    gboolean play_sound_enabled;
    GtkTreeSelection *selection;
    gboolean has_selection;
    gboolean popups_enabled;
    gboolean popups_expiration_after_enabled;

    play_sound_enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(selfp->play_sound_check));

    gtk_widget_set_sensitive(selfp->play_sound_file_chooser_button, play_sound_enabled);

    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(selfp->list));
    has_selection = gtk_tree_selection_count_selected_rows(selection) > 0;

    gtk_widget_set_sensitive(selfp->remove, has_selection);
    gtk_widget_set_sensitive(selfp->properties, has_selection);

    popups_enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(selfp->popups_enable_check));
    popups_expiration_after_enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(selfp->popups_expiration_after_radio));

    gtk_widget_set_sensitive(selfp->popups_position_section_label, popups_enabled);
    gtk_widget_set_sensitive(selfp->popups_position_attached_radio, popups_enabled);
    gtk_widget_set_sensitive(selfp->popups_position_free_radio, popups_enabled);
    gtk_widget_set_sensitive(selfp->popups_expiration_section_label, popups_enabled);
    gtk_widget_set_sensitive(selfp->popups_expiration_default_radio, popups_enabled);
    gtk_widget_set_sensitive(selfp->popups_expiration_never_radio, popups_enabled);
    gtk_widget_set_sensitive(selfp->popups_expiration_after_radio, popups_enabled);
    gtk_widget_set_sensitive(selfp->popups_expiration_scale, popups_enabled && popups_expiration_after_enabled);
  }}
#line 440 "mn-properties-dialog.c"
#undef __GOB_FUNCTION__

#line 279 "src/mn-properties-dialog.gob"
void 
mn_properties_dialog_toggled_h (MNPropertiesDialog * self, GtkToggleButton * button)
{
#line 447 "mn-properties-dialog.c"
#define __GOB_FUNCTION__ "MN:Properties:Dialog::toggled_h"
#line 279 "src/mn-properties-dialog.gob"
	g_return_if_fail (self != NULL);
#line 279 "src/mn-properties-dialog.gob"
	g_return_if_fail (MN_IS_PROPERTIES_DIALOG (self));
#line 453 "mn-properties-dialog.c"
{
#line 281 "src/mn-properties-dialog.gob"
	
    self_update_sensitivity(self);
  }}
#line 459 "mn-properties-dialog.c"
#undef __GOB_FUNCTION__

#line 285 "src/mn-properties-dialog.gob"
void 
mn_properties_dialog_add_clicked_h (MNPropertiesDialog * self, GtkButton * button)
{
#line 466 "mn-properties-dialog.c"
#define __GOB_FUNCTION__ "MN:Properties:Dialog::add_clicked_h"
#line 285 "src/mn-properties-dialog.gob"
	g_return_if_fail (self != NULL);
#line 285 "src/mn-properties-dialog.gob"
	g_return_if_fail (MN_IS_PROPERTIES_DIALOG (self));
#line 472 "mn-properties-dialog.c"
{
#line 287 "src/mn-properties-dialog.gob"
	
    mn_mailbox_view_activate_add(MN_MAILBOX_VIEW(selfp->list));
  }}
#line 478 "mn-properties-dialog.c"
#undef __GOB_FUNCTION__

#line 291 "src/mn-properties-dialog.gob"
void 
mn_properties_dialog_remove_clicked_h (MNPropertiesDialog * self, GtkButton * button)
{
#line 485 "mn-properties-dialog.c"
#define __GOB_FUNCTION__ "MN:Properties:Dialog::remove_clicked_h"
#line 291 "src/mn-properties-dialog.gob"
	g_return_if_fail (self != NULL);
#line 291 "src/mn-properties-dialog.gob"
	g_return_if_fail (MN_IS_PROPERTIES_DIALOG (self));
#line 491 "mn-properties-dialog.c"
{
#line 293 "src/mn-properties-dialog.gob"
	
    mn_mailbox_view_activate_remove(MN_MAILBOX_VIEW(selfp->list));
  }}
#line 497 "mn-properties-dialog.c"
#undef __GOB_FUNCTION__

#line 297 "src/mn-properties-dialog.gob"
void 
mn_properties_dialog_properties_clicked_h (MNPropertiesDialog * self, GtkButton * button)
{
#line 504 "mn-properties-dialog.c"
#define __GOB_FUNCTION__ "MN:Properties:Dialog::properties_clicked_h"
#line 297 "src/mn-properties-dialog.gob"
	g_return_if_fail (self != NULL);
#line 297 "src/mn-properties-dialog.gob"
	g_return_if_fail (MN_IS_PROPERTIES_DIALOG (self));
#line 510 "mn-properties-dialog.c"
{
#line 299 "src/mn-properties-dialog.gob"
	
    mn_mailbox_view_activate_properties(MN_MAILBOX_VIEW(selfp->list));
  }}
#line 516 "mn-properties-dialog.c"
#undef __GOB_FUNCTION__

#line 303 "src/mn-properties-dialog.gob"
char * 
mn_properties_dialog_popups_expiration_format_value_h (MNPropertiesDialog * self, double arg, GtkScale * scale)
{
#line 523 "mn-properties-dialog.c"
#define __GOB_FUNCTION__ "MN:Properties:Dialog::popups_expiration_format_value_h"
#line 303 "src/mn-properties-dialog.gob"
	g_return_val_if_fail (self != NULL, (char * )0);
#line 303 "src/mn-properties-dialog.gob"
	g_return_val_if_fail (MN_IS_PROPERTIES_DIALOG (self), (char * )0);
#line 529 "mn-properties-dialog.c"
{
#line 305 "src/mn-properties-dialog.gob"
	
    return mn_format_seconds(mn_non_linear_range_get_value(GTK_RANGE(scale)));
  }}
#line 535 "mn-properties-dialog.c"
#undef __GOB_FUNCTION__

#line 309 "src/mn-properties-dialog.gob"
void 
mn_properties_dialog_test_messages_toggled_h (MNPropertiesDialog * self, GtkToggleButton * button)
{
#line 542 "mn-properties-dialog.c"
#define __GOB_FUNCTION__ "MN:Properties:Dialog::test_messages_toggled_h"
#line 309 "src/mn-properties-dialog.gob"
	g_return_if_fail (self != NULL);
#line 309 "src/mn-properties-dialog.gob"
	g_return_if_fail (MN_IS_PROPERTIES_DIALOG (self));
#line 548 "mn-properties-dialog.c"
{
#line 311 "src/mn-properties-dialog.gob"
	
    if (gtk_toggle_button_get_active(button))
      {
	if (! selfp->test_mailbox)
	  {
	    selfp->test_mailbox = mn_test_mailbox_new();
	    mn_mailbox_seal(selfp->test_mailbox);
	    mn_mailboxes_add(mn_shell->mailboxes, selfp->test_mailbox);
	  }
      }
    else
      self_remove_test_mailbox(self);
  }}
#line 564 "mn-properties-dialog.c"
#undef __GOB_FUNCTION__
