/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_POPUP_PRIVATE_H__
#define __MN_POPUP_PRIVATE_H__

#include "mn-popup.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

struct _MNPopupPrivate {
#line 34 "src/mn-popup.gob"
	MNMessage * message;
#line 16 "mn-popup-private.h"
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
