/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_AUTHENTICATED_MAILBOX_PROPERTIES_PRIVATE_H__
#define __MN_AUTHENTICATED_MAILBOX_PROPERTIES_PRIVATE_H__

#include "mn-authenticated-mailbox-properties.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#line 25 "src/mn-authenticated-mailbox-properties.gob"

#include "mn-authenticated-mailbox.h"

#line 18 "mn-authenticated-mailbox-properties-private.h"
struct _MNAuthenticatedMailboxPropertiesPrivate {
#line 44 "src/mn-authenticated-mailbox-properties.gob"
	GtkWidget * password_operation_label;
#line 45 "src/mn-authenticated-mailbox-properties.gob"
	GtkWidget * password_widget;
#line 47 "src/mn-authenticated-mailbox-properties.gob"
	MNAuthenticatedMailbox * mailbox_setting_password;
#line 48 "src/mn-authenticated-mailbox-properties.gob"
	gpointer get_password_request;
#line 28 "mn-authenticated-mailbox-properties-private.h"
};
#line 93 "src/mn-authenticated-mailbox-properties.gob"
GtkWidget * 	mn_authenticated_mailbox_properties_field_new	(MNAuthenticatedMailboxProperties * self, const char * mnemonic, GtkWidget ** label, GtkWidget ** entry);
#line 32 "mn-authenticated-mailbox-properties-private.h"
#line 270 "src/mn-authenticated-mailbox-properties.gob"
void 	mn_authenticated_mailbox_properties_get_contents	(MNAuthenticatedMailboxProperties * self, const char ** username, const char ** password);
#line 35 "mn-authenticated-mailbox-properties-private.h"
#line 291 "src/mn-authenticated-mailbox-properties.gob"
gboolean 	mn_authenticated_mailbox_properties_is_complete	(MNAuthenticatedMailboxProperties * self);
#line 38 "mn-authenticated-mailbox-properties-private.h"

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
