/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_TEXT_TABLE_PRIVATE_H__
#define __MN_TEXT_TABLE_PRIVATE_H__

#include "mn-text-table.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#line 28 "src/mn-text-table.gob"

typedef struct
{
  GPtrArray	*cells;
  int		height;
} Row;

#line 22 "mn-text-table-private.h"
struct _MNTextTablePrivate {
#line 67 "src/mn-text-table.gob"
	GPtrArray * rows;
#line 70 "src/mn-text-table.gob"
	GArray * column_sizes;
#line 73 "src/mn-text-table.gob"
	Row * row;
#line 75 "src/mn-text-table.gob"
	gboolean dirty;
#line 76 "src/mn-text-table.gob"
	int width;
#line 77 "src/mn-text-table.gob"
	int height;
#line 36 "mn-text-table-private.h"
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
