/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_SHELL_PRIVATE_H__
#define __MN_SHELL_PRIVATE_H__

#include "mn-shell.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#line 29 "src/mn-shell.gob"

#include "mn-sound-player.h"

#line 18 "mn-shell-private.h"
struct _MNShellPrivate {
#line 89 "src/mn-shell.gob"
	int unix_signal_pipe_write_end;
#line 91 "src/mn-shell.gob"
	MNSoundPlayer * sound_player;
#line 93 "src/mn-shell.gob"
	gboolean has_new;
#line 95 "src/mn-shell.gob"
	GtkWidget * properties_dialog;
#line 96 "src/mn-shell.gob"
	GSList * mailbox_properties_dialogs;
#line 97 "src/mn-shell.gob"
	GtkWidget * about_dialog;
#line 32 "mn-shell-private.h"
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
