/* Generated by GOB (v2.0.15)   (do not edit directly) */

#include <glib.h>
#include <glib-object.h>


#include <gtk/gtk.h>

#ifndef __MN_TEXT_TABLE_H__
#define __MN_TEXT_TABLE_H__

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */



typedef struct _MNTextTableCell MNTextTableCell;


/*
 * Type checking and casting macros
 */
#define MN_TYPE_TEXT_TABLE	(mn_text_table_get_type())
#define MN_TEXT_TABLE(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), mn_text_table_get_type(), MNTextTable)
#define MN_TEXT_TABLE_CONST(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), mn_text_table_get_type(), MNTextTable const)
#define MN_TEXT_TABLE_CLASS(klass)	G_TYPE_CHECK_CLASS_CAST((klass), mn_text_table_get_type(), MNTextTableClass)
#define MN_IS_TEXT_TABLE(obj)	G_TYPE_CHECK_INSTANCE_TYPE((obj), mn_text_table_get_type ())

#define MN_TEXT_TABLE_GET_CLASS(obj)	G_TYPE_INSTANCE_GET_CLASS((obj), mn_text_table_get_type(), MNTextTableClass)

/* Private structure type */
typedef struct _MNTextTablePrivate MNTextTablePrivate;

/*
 * Main object structure
 */
#ifndef __TYPEDEF_MN_TEXT_TABLE__
#define __TYPEDEF_MN_TEXT_TABLE__
typedef struct _MNTextTable MNTextTable;
#endif
struct _MNTextTable {
	GtkWidget __parent__;
	/*< private >*/
	MNTextTablePrivate *_priv;
};

/*
 * Class definition
 */
typedef struct _MNTextTableClass MNTextTableClass;
struct _MNTextTableClass {
	GtkWidgetClass __parent__;
	void (* clear) (MNTextTable * self);
};


/*
 * Public methods
 */
GType	mn_text_table_get_type	(void) G_GNUC_CONST;
#line 257 "src/mn-text-table.gob"
void 	mn_text_table_clear	(MNTextTable * self);
#line 65 "mn-text-table.h"
#line 270 "src/mn-text-table.gob"
MNTextTableCell * 	mn_text_table_append_text_cell	(MNTextTable * self,
					const char * text);
#line 69 "mn-text-table.h"
#line 276 "src/mn-text-table.gob"
MNTextTableCell * 	mn_text_table_append_text_cell_from_markup	(MNTextTable * self,
					const char * markup);
#line 73 "mn-text-table.h"
#line 287 "src/mn-text-table.gob"
MNTextTableCell * 	mn_text_table_append_text_cell_from_layout	(MNTextTable * self,
					PangoLayout * layout);
#line 77 "mn-text-table.h"
#line 301 "src/mn-text-table.gob"
MNTextTableCell * 	mn_text_table_append_blank_cell	(MNTextTable * self,
					int width,
					int height);
#line 82 "mn-text-table.h"
#line 333 "src/mn-text-table.gob"
void 	mn_text_table_cell_set_text	(MNTextTable * self,
					MNTextTableCell * cell,
					const char * text);
#line 87 "mn-text-table.h"
#line 351 "src/mn-text-table.gob"
void 	mn_text_table_line_break	(MNTextTable * self);
#line 90 "mn-text-table.h"

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
