/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_GMAIL_MAILBOX_PROPERTIES_PRIVATE_H__
#define __MN_GMAIL_MAILBOX_PROPERTIES_PRIVATE_H__

#include "mn-gmail-mailbox-properties.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

struct _MNGmailMailboxPropertiesPrivate {
#line 39 "src/mn-gmail-mailbox-properties.gob"
	GtkWidget * label_check;
#line 40 "src/mn-gmail-mailbox-properties.gob"
	GtkWidget * label_entry;
#line 46 "src/mn-gmail-mailbox-properties.gob"
	char * location;
#line 20 "mn-gmail-mailbox-properties-private.h"
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
