/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_MAILBOXES_PRIVATE_H__
#define __MN_MAILBOXES_PRIVATE_H__

#include "mn-mailboxes.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

struct _MNMailboxesPrivate {
#line 52 "src/mn-mailboxes.gob"
	GSList * add_queue;
#line 53 "src/mn-mailboxes.gob"
	GSList * remove_queue;
#line 54 "src/mn-mailboxes.gob"
	unsigned int queue_idle_id;
#line 20 "mn-mailboxes-private.h"
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
