/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_MAILDIR_MESSAGE_PRIVATE_H__
#define __MN_MAILDIR_MESSAGE_PRIVATE_H__

#include "mn-maildir-message.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#line 24 "src/mn-maildir-message.gob"

#include <gtk/gtk.h>

#line 18 "mn-maildir-message-private.h"

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
