/* Generated by GOB (v2.0.15)   (do not edit directly) */

#include <glib.h>
#include <glib-object.h>


#include <libnotify/notify.h>
#include "mn-message.h"

#ifndef __MN_POPUP_H__
#define __MN_POPUP_H__

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/*
 * Type checking and casting macros
 */
#define MN_TYPE_POPUP	(mn_popup_get_type())
#define MN_POPUP(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), mn_popup_get_type(), MNPopup)
#define MN_POPUP_CONST(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), mn_popup_get_type(), MNPopup const)
#define MN_POPUP_CLASS(klass)	G_TYPE_CHECK_CLASS_CAST((klass), mn_popup_get_type(), MNPopupClass)
#define MN_IS_POPUP(obj)	G_TYPE_CHECK_INSTANCE_TYPE((obj), mn_popup_get_type ())

#define MN_POPUP_GET_CLASS(obj)	G_TYPE_INSTANCE_GET_CLASS((obj), mn_popup_get_type(), MNPopupClass)

/* Private structure type */
typedef struct _MNPopupPrivate MNPopupPrivate;

/*
 * Main object structure
 */
#ifndef __TYPEDEF_MN_POPUP__
#define __TYPEDEF_MN_POPUP__
typedef struct _MNPopup MNPopup;
#endif
struct _MNPopup {
	NotifyNotification __parent__;
	/*< public >*/
	gboolean visible;
	GTimeVal show_timestamp;
	/*< private >*/
	MNPopupPrivate *_priv;
};

/*
 * Class definition
 */
typedef struct _MNPopupClass MNPopupClass;
struct _MNPopupClass {
	NotifyNotificationClass __parent__;
};


/*
 * Public methods
 */
GType	mn_popup_get_type	(void) G_GNUC_CONST;
#line 166 "src/mn-popup.gob"
void 	mn_popup_show	(MNPopup * self);
#line 64 "mn-popup.h"
#line 184 "src/mn-popup.gob"
void 	mn_popup_close	(MNPopup * self);
#line 67 "mn-popup.h"
#line 199 "src/mn-popup.gob"
MNPopup * 	mn_popup_new	(MNMessage * message);
#line 70 "mn-popup.h"

/*
 * Argument wrapping macros
 */
#if defined(__GNUC__) && !defined(__STRICT_ANSI__)
#define MN_POPUP_PROP_MESSAGE(arg)    	"message", __extension__ ({MNMessage * z = (arg); z;})
#define MN_POPUP_GET_PROP_MESSAGE(arg)	"message", __extension__ ({MNMessage * *z = (arg); z;})
#else /* __GNUC__ && !__STRICT_ANSI__ */
#define MN_POPUP_PROP_MESSAGE(arg)    	"message",(MNMessage * )(arg)
#define MN_POPUP_GET_PROP_MESSAGE(arg)	"message",(MNMessage * *)(arg)
#endif /* __GNUC__ && !__STRICT_ANSI__ */


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
