/* Generated by GOB (v2.0.15)   (do not edit directly) */

#include <glib.h>
#include <glib-object.h>


#include "mn-dialog.h"

#ifndef __MN_PROPERTIES_DIALOG_H__
#define __MN_PROPERTIES_DIALOG_H__

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/*
 * Type checking and casting macros
 */
#define MN_TYPE_PROPERTIES_DIALOG	(mn_properties_dialog_get_type())
#define MN_PROPERTIES_DIALOG(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), mn_properties_dialog_get_type(), MNPropertiesDialog)
#define MN_PROPERTIES_DIALOG_CONST(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), mn_properties_dialog_get_type(), MNPropertiesDialog const)
#define MN_PROPERTIES_DIALOG_CLASS(klass)	G_TYPE_CHECK_CLASS_CAST((klass), mn_properties_dialog_get_type(), MNPropertiesDialogClass)
#define MN_IS_PROPERTIES_DIALOG(obj)	G_TYPE_CHECK_INSTANCE_TYPE((obj), mn_properties_dialog_get_type ())

#define MN_PROPERTIES_DIALOG_GET_CLASS(obj)	G_TYPE_INSTANCE_GET_CLASS((obj), mn_properties_dialog_get_type(), MNPropertiesDialogClass)

/* Private structure type */
typedef struct _MNPropertiesDialogPrivate MNPropertiesDialogPrivate;

/*
 * Main object structure
 */
#ifndef __TYPEDEF_MN_PROPERTIES_DIALOG__
#define __TYPEDEF_MN_PROPERTIES_DIALOG__
typedef struct _MNPropertiesDialog MNPropertiesDialog;
#endif
struct _MNPropertiesDialog {
	MNDialog __parent__;
	/*< private >*/
	MNPropertiesDialogPrivate *_priv;
};

/*
 * Class definition
 */
typedef struct _MNPropertiesDialogClass MNPropertiesDialogClass;
struct _MNPropertiesDialogClass {
	MNDialogClass __parent__;
};


/*
 * Public methods
 */
GType	mn_properties_dialog_get_type	(void) G_GNUC_CONST;

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
