/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_MAILBOX_VIEW_PRIVATE_H__
#define __MN_MAILBOX_VIEW_PRIVATE_H__

#include "mn-mailbox-view.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

struct _MNMailboxViewPrivate {
#line 75 "src/mn-mailbox-view.gob"
	GtkWidget * menu;
#line 76 "src/mn-mailbox-view.gob"
	GtkWidget * properties_item;
#line 77 "src/mn-mailbox-view.gob"
	GtkWidget * update_item;
#line 78 "src/mn-mailbox-view.gob"
	GtkWidget * remove_item;
#line 79 "src/mn-mailbox-view.gob"
	GtkWidget * cut_item;
#line 80 "src/mn-mailbox-view.gob"
	GtkWidget * copy_item;
#line 81 "src/mn-mailbox-view.gob"
	GtkWidget * paste_item;
#line 82 "src/mn-mailbox-view.gob"
	GtkWidget * select_all_item;
#line 30 "mn-mailbox-view-private.h"
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
