/* Generated by GOB (v2.0.15)   (do not edit directly) */

#include <glib.h>
#include <glib-object.h>


#include "mn-vfs-mailbox.h"

#ifndef __MN_VFS_MAILBOX_BACKEND_H__
#define __MN_VFS_MAILBOX_BACKEND_H__

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/*
 * Type checking and casting macros
 */
#define MN_TYPE_VFS_MAILBOX_BACKEND	(mn_vfs_mailbox_backend_get_type())
#define MN_VFS_MAILBOX_BACKEND(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), mn_vfs_mailbox_backend_get_type(), MNVFSMailboxBackend)
#define MN_VFS_MAILBOX_BACKEND_CONST(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), mn_vfs_mailbox_backend_get_type(), MNVFSMailboxBackend const)
#define MN_VFS_MAILBOX_BACKEND_CLASS(klass)	G_TYPE_CHECK_CLASS_CAST((klass), mn_vfs_mailbox_backend_get_type(), MNVFSMailboxBackendClass)
#define MN_IS_VFS_MAILBOX_BACKEND(obj)	G_TYPE_CHECK_INSTANCE_TYPE((obj), mn_vfs_mailbox_backend_get_type ())

#define MN_VFS_MAILBOX_BACKEND_GET_CLASS(obj)	G_TYPE_INSTANCE_GET_CLASS((obj), mn_vfs_mailbox_backend_get_type(), MNVFSMailboxBackendClass)

/* Private structure type */
typedef struct _MNVFSMailboxBackendPrivate MNVFSMailboxBackendPrivate;

/*
 * Main object structure
 */
#ifndef __TYPEDEF_MN_VFS_MAILBOX_BACKEND__
#define __TYPEDEF_MN_VFS_MAILBOX_BACKEND__
typedef struct _MNVFSMailboxBackend MNVFSMailboxBackend;
#endif
struct _MNVFSMailboxBackend {
	GObject __parent__;
	/*< private >*/
	MNVFSMailbox * mailbox; /* protected */
	int check_latency; /* protected */
	MNVFSMailboxBackendPrivate *_priv;
};

/*
 * Class definition
 */
typedef struct _MNVFSMailboxBackendClass MNVFSMailboxBackendClass;
struct _MNVFSMailboxBackendClass {
	GObjectClass __parent__;
	void (* monitor_cb) (MNVFSMailboxBackend * self, const char * info_uri, GnomeVFSMonitorEventType event_type);
	gboolean (* is) (MNVFSMailboxBackend * self, MNVFSMailboxBackendClass * class, MNVFSMailbox * mailbox);
	void (* check) (MNVFSMailboxBackend * self, int check_id);
	const char * format;
};


/*
 * Public methods
 */
GType	mn_vfs_mailbox_backend_get_type	(void) G_GNUC_CONST;
#line 135 "src/mn-vfs-mailbox-backend.gob"
gboolean 	mn_vfs_mailbox_backend_is	(MNVFSMailboxBackend * self,
					MNVFSMailboxBackendClass * class,
					MNVFSMailbox * mailbox);
#line 68 "mn-vfs-mailbox-backend.h"
#line 138 "src/mn-vfs-mailbox-backend.gob"
void 	mn_vfs_mailbox_backend_check	(MNVFSMailboxBackend * self,
					int check_id);
#line 72 "mn-vfs-mailbox-backend.h"

/*
 * Argument wrapping macros
 */
#if defined(__GNUC__) && !defined(__STRICT_ANSI__)
#define MN_VFS_MAILBOX_BACKEND_PROP_MAILBOX(arg)    	"mailbox", __extension__ ({MNVFSMailbox * z = (arg); z;})
#define MN_VFS_MAILBOX_BACKEND_GET_PROP_MAILBOX(arg)	"mailbox", __extension__ ({MNVFSMailbox * *z = (arg); z;})
#else /* __GNUC__ && !__STRICT_ANSI__ */
#define MN_VFS_MAILBOX_BACKEND_PROP_MAILBOX(arg)    	"mailbox",(MNVFSMailbox * )(arg)
#define MN_VFS_MAILBOX_BACKEND_GET_PROP_MAILBOX(arg)	"mailbox",(MNVFSMailbox * *)(arg)
#endif /* __GNUC__ && !__STRICT_ANSI__ */


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
