/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_IMAP_MAILBOX_PRIVATE_H__
#define __MN_IMAP_MAILBOX_PRIVATE_H__

#include "mn-imap-mailbox.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#line 38 "src/mn-imap-mailbox.gob"

#include "mn-client-session.h"

#line 18 "mn-imap-mailbox-private.h"
struct _MNIMAPMailboxPrivate {
#line 160 "src/mn-imap-mailbox.gob"
	GMutex * mutex;
#line 162 "src/mn-imap-mailbox.gob"
	MNClientSessionPrivate * idle_session;
#line 24 "mn-imap-mailbox-private.h"
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
