/* Generated by GOB (v2.0.15)   (do not edit directly) */

#include <glib.h>
#include <glib-object.h>
#ifndef __MN_EVOLUTION_FOLDER_TREE_SERVER_H__
#define __MN_EVOLUTION_FOLDER_TREE_SERVER_H__

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/*
 * Type checking and casting macros
 */
#define MN_TYPE_EVOLUTION_FOLDER_TREE_SERVER	(mn_evolution_folder_tree_server_get_type())
#define MN_EVOLUTION_FOLDER_TREE_SERVER(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), mn_evolution_folder_tree_server_get_type(), MNEvolutionFolderTreeServer)
#define MN_EVOLUTION_FOLDER_TREE_SERVER_CONST(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), mn_evolution_folder_tree_server_get_type(), MNEvolutionFolderTreeServer const)
#define MN_EVOLUTION_FOLDER_TREE_SERVER_CLASS(klass)	G_TYPE_CHECK_CLASS_CAST((klass), mn_evolution_folder_tree_server_get_type(), MNEvolutionFolderTreeServerClass)
#define MN_IS_EVOLUTION_FOLDER_TREE_SERVER(obj)	G_TYPE_CHECK_INSTANCE_TYPE((obj), mn_evolution_folder_tree_server_get_type ())

#define MN_EVOLUTION_FOLDER_TREE_SERVER_GET_CLASS(obj)	G_TYPE_INSTANCE_GET_CLASS((obj), mn_evolution_folder_tree_server_get_type(), MNEvolutionFolderTreeServerClass)

/* Private structure type */
typedef struct _MNEvolutionFolderTreeServerPrivate MNEvolutionFolderTreeServerPrivate;

/*
 * Main object structure
 */
#ifndef __TYPEDEF_MN_EVOLUTION_FOLDER_TREE_SERVER__
#define __TYPEDEF_MN_EVOLUTION_FOLDER_TREE_SERVER__
typedef struct _MNEvolutionFolderTreeServer MNEvolutionFolderTreeServer;
#endif
struct _MNEvolutionFolderTreeServer {
	GObject __parent__;
	/*< private >*/
	MNEvolutionFolderTreeServerPrivate *_priv;
};

/*
 * Class definition
 */
typedef struct _MNEvolutionFolderTreeServerClass MNEvolutionFolderTreeServerClass;
struct _MNEvolutionFolderTreeServerClass {
	GObjectClass __parent__;
	/*signal*/void (* folder_selected) (MNEvolutionFolderTreeServer * self, const char * uri);
	/*signal*/void (* folder_activated) (MNEvolutionFolderTreeServer * self);
};


/*
 * Public methods
 */
GType	mn_evolution_folder_tree_server_get_type	(void) G_GNUC_CONST;
#line 125 "src/mn-evolution-folder-tree-server.gob"
MNEvolutionFolderTreeServer * 	mn_evolution_folder_tree_server_new	(guint32 id);
#line 58 "mn-evolution-folder-tree-server.h"

/*
 * Signal connection wrapper macros
 */
#if defined(__GNUC__) && !defined(__STRICT_ANSI__)
#define mn_evolution_folder_tree_server_connect__folder_selected(object,func,data)	g_signal_connect(MN_EVOLUTION_FOLDER_TREE_SERVER(__extension__ ({MNEvolutionFolderTreeServer *___object = (object); ___object; })),"folder_selected",(GCallback) __extension__ ({void (* ___folder_selected) (MNEvolutionFolderTreeServer * ___fake___self, const char * ___fake___uri, gpointer ___data ) = (func); ___folder_selected; }), (data))
#define mn_evolution_folder_tree_server_connect_after__folder_selected(object,func,data)	g_signal_connect_after(MN_EVOLUTION_FOLDER_TREE_SERVER(__extension__ ({MNEvolutionFolderTreeServer *___object = (object); ___object; })),"folder_selected",(GCallback) __extension__ ({void (* ___folder_selected) (MNEvolutionFolderTreeServer * ___fake___self, const char * ___fake___uri, gpointer ___data ) = (func); ___folder_selected; }), (data))
#define mn_evolution_folder_tree_server_connect_data__folder_selected(object,func,data,destroy_data,flags)	g_signal_connect_data(MN_EVOLUTION_FOLDER_TREE_SERVER(__extension__ ({MNEvolutionFolderTreeServer *___object = (object); ___object; })),"folder_selected",(GCallback) __extension__ ({void (* ___folder_selected) (MNEvolutionFolderTreeServer * ___fake___self, const char * ___fake___uri, gpointer ___data ) = (func); ___folder_selected; }), (data), (destroy_data), (GConnectFlags)(flags))
#define mn_evolution_folder_tree_server_connect__folder_activated(object,func,data)	g_signal_connect(MN_EVOLUTION_FOLDER_TREE_SERVER(__extension__ ({MNEvolutionFolderTreeServer *___object = (object); ___object; })),"folder_activated",(GCallback) __extension__ ({void (* ___folder_activated) (MNEvolutionFolderTreeServer * ___fake___self, gpointer ___data ) = (func); ___folder_activated; }), (data))
#define mn_evolution_folder_tree_server_connect_after__folder_activated(object,func,data)	g_signal_connect_after(MN_EVOLUTION_FOLDER_TREE_SERVER(__extension__ ({MNEvolutionFolderTreeServer *___object = (object); ___object; })),"folder_activated",(GCallback) __extension__ ({void (* ___folder_activated) (MNEvolutionFolderTreeServer * ___fake___self, gpointer ___data ) = (func); ___folder_activated; }), (data))
#define mn_evolution_folder_tree_server_connect_data__folder_activated(object,func,data,destroy_data,flags)	g_signal_connect_data(MN_EVOLUTION_FOLDER_TREE_SERVER(__extension__ ({MNEvolutionFolderTreeServer *___object = (object); ___object; })),"folder_activated",(GCallback) __extension__ ({void (* ___folder_activated) (MNEvolutionFolderTreeServer * ___fake___self, gpointer ___data ) = (func); ___folder_activated; }), (data), (destroy_data), (GConnectFlags)(flags))
#else /* __GNUC__ && !__STRICT_ANSI__ */
#define mn_evolution_folder_tree_server_connect__folder_selected(object,func,data)	g_signal_connect(MN_EVOLUTION_FOLDER_TREE_SERVER(object),"folder_selected",(GCallback)(func),(data))
#define mn_evolution_folder_tree_server_connect_after__folder_selected(object,func,data)	g_signal_connect_after(MN_EVOLUTION_FOLDER_TREE_SERVER(object),"folder_selected",(GCallback)(func),(data))
#define mn_evolution_folder_tree_server_connect_data__folder_selected(object,func,data,destroy_data,flags)	g_signal_connect_data(MN_EVOLUTION_FOLDER_TREE_SERVER(object),"folder_selected",(GCallback)(func),(data),(destroy_data),(GConnectFlags)(flags))
#define mn_evolution_folder_tree_server_connect__folder_activated(object,func,data)	g_signal_connect(MN_EVOLUTION_FOLDER_TREE_SERVER(object),"folder_activated",(GCallback)(func),(data))
#define mn_evolution_folder_tree_server_connect_after__folder_activated(object,func,data)	g_signal_connect_after(MN_EVOLUTION_FOLDER_TREE_SERVER(object),"folder_activated",(GCallback)(func),(data))
#define mn_evolution_folder_tree_server_connect_data__folder_activated(object,func,data,destroy_data,flags)	g_signal_connect_data(MN_EVOLUTION_FOLDER_TREE_SERVER(object),"folder_activated",(GCallback)(func),(data),(destroy_data),(GConnectFlags)(flags))
#endif /* __GNUC__ && !__STRICT_ANSI__ */


/*
 * Argument wrapping macros
 */
#if defined(__GNUC__) && !defined(__STRICT_ANSI__)
#define MN_EVOLUTION_FOLDER_TREE_SERVER_PROP_ID(arg)    	"id", __extension__ ({guint z = (arg); z;})
#define MN_EVOLUTION_FOLDER_TREE_SERVER_GET_PROP_ID(arg)	"id", __extension__ ({guint *z = (arg); z;})
#define MN_EVOLUTION_FOLDER_TREE_SERVER_PROP_URI(arg)    	"uri", __extension__ ({gchar *z = (arg); z;})
#else /* __GNUC__ && !__STRICT_ANSI__ */
#define MN_EVOLUTION_FOLDER_TREE_SERVER_PROP_ID(arg)    	"id",(guint )(arg)
#define MN_EVOLUTION_FOLDER_TREE_SERVER_GET_PROP_ID(arg)	"id",(guint *)(arg)
#define MN_EVOLUTION_FOLDER_TREE_SERVER_PROP_URI(arg)    	"uri",(gchar *)(arg)
#endif /* __GNUC__ && !__STRICT_ANSI__ */


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
