/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_EVOLUTION_FOLDER_TREE_CLIENT_PRIVATE_H__
#define __MN_EVOLUTION_FOLDER_TREE_CLIENT_PRIVATE_H__

#include "mn-evolution-folder-tree-client.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#line 24 "src/mn-evolution-folder-tree-client.gob"

#include "mn-evolution-client.h"

#line 18 "mn-evolution-folder-tree-client-private.h"
struct _MNEvolutionFolderTreeClientPrivate {
#line 39 "src/mn-evolution-folder-tree-client.gob"
	DBusGProxy * proxy;
#line 40 "src/mn-evolution-folder-tree-client.gob"
	DBusGProxy * properties_proxy;
#line 63 "src/mn-evolution-folder-tree-client.gob"
	MNEvolutionClient * client;
#line 65 "src/mn-evolution-folder-tree-client.gob"
	unsigned int contact_error_idle_id;
#line 28 "mn-evolution-folder-tree-client-private.h"
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
