/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_MAIL_ICON_PRIVATE_H__
#define __MN_MAIL_ICON_PRIVATE_H__

#include "mn-mail-icon.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#line 31 "src/mn-mail-icon.gob"

#include "mn-tooltips.h"

#line 18 "mn-mail-icon-private.h"
struct _MNMailIconPrivate {
#line 45 "src/mn-mail-icon.gob"
	MNTooltips * tooltips;
#line 46 "src/mn-mail-icon.gob"
	GtkWidget * widget;
#line 48 "src/mn-mail-icon.gob"
	char * stock_id;
#line 49 "src/mn-mail-icon.gob"
	int size;
#line 51 "src/mn-mail-icon.gob"
	GtkWidget * menu;
#line 30 "mn-mail-icon-private.h"
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
