/* Generated by GOB (v2.0.15)   (do not edit directly) */

#include <glib.h>
#include <glib-object.h>


#include <dbus/dbus-glib.h>

#ifndef __MN_SERVER_H__
#define __MN_SERVER_H__

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */



#define MN_SERVER_SERVICE	"org.gnome.MailNotification"
#define MN_SERVER_PATH		"/org/gnome/MailNotification"
#define MN_SERVER_INTERFACE	"org.gnome.MailNotification"


/*
 * Type checking and casting macros
 */
#define MN_TYPE_SERVER	(mn_server_get_type())
#define MN_SERVER(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), mn_server_get_type(), MNServer)
#define MN_SERVER_CONST(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), mn_server_get_type(), MNServer const)
#define MN_SERVER_CLASS(klass)	G_TYPE_CHECK_CLASS_CAST((klass), mn_server_get_type(), MNServerClass)
#define MN_IS_SERVER(obj)	G_TYPE_CHECK_INSTANCE_TYPE((obj), mn_server_get_type ())

#define MN_SERVER_GET_CLASS(obj)	G_TYPE_INSTANCE_GET_CLASS((obj), mn_server_get_type(), MNServerClass)

/*
 * Main object structure
 */
#ifndef __TYPEDEF_MN_SERVER__
#define __TYPEDEF_MN_SERVER__
typedef struct _MNServer MNServer;
#endif
struct _MNServer {
	GObject __parent__;
};

/*
 * Class definition
 */
typedef struct _MNServerClass MNServerClass;
struct _MNServerClass {
	GObjectClass __parent__;
};


/*
 * Public methods
 */
GType	mn_server_get_type	(void) G_GNUC_CONST;
#line 118 "src/mn-server.gob"
gboolean 	mn_server_start	(DBusGConnection * bus,
					DBusGProxy * bus_proxy);
#line 62 "mn-server.h"

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
