/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_SOUND_PLAYER_PRIVATE_H__
#define __MN_SOUND_PLAYER_PRIVATE_H__

#include "mn-sound-player.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

struct _MNSoundPlayerPrivate {
#line 36 "src/mn-sound-player.gob"
	gboolean can_play;
#line 49 "src/mn-sound-player.gob"
	GPid pid;
#line 50 "src/mn-sound-player.gob"
	GSource * watch_source;
#line 20 "mn-sound-player-private.h"
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
