/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_SOUND_FILE_CHOOSER_DIALOG_PRIVATE_H__
#define __MN_SOUND_FILE_CHOOSER_DIALOG_PRIVATE_H__

#include "mn-sound-file-chooser-dialog.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#line 24 "src/mn-sound-file-chooser-dialog.gob"

#include "mn-sound-player.h"

#line 18 "mn-sound-file-chooser-dialog-private.h"
struct _MNSoundFileChooserDialogPrivate {
#line 40 "src/mn-sound-file-chooser-dialog.gob"
	MNSoundPlayer * sound_player;
#line 22 "mn-sound-file-chooser-dialog-private.h"
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
