/* Generated by GOB (v2.0.15)   (do not edit directly) */

#include <glib.h>
#include <glib-object.h>


#include <gtk/gtk.h>
#include "eggtrayicon.h"

#ifndef __MN_MAIL_ICON_H__
#define __MN_MAIL_ICON_H__

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/*
 * Type checking and casting macros
 */
#define MN_TYPE_MAIL_ICON	(mn_mail_icon_get_type())
#define MN_MAIL_ICON(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), mn_mail_icon_get_type(), MNMailIcon)
#define MN_MAIL_ICON_CONST(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), mn_mail_icon_get_type(), MNMailIcon const)
#define MN_MAIL_ICON_CLASS(klass)	G_TYPE_CHECK_CLASS_CAST((klass), mn_mail_icon_get_type(), MNMailIconClass)
#define MN_IS_MAIL_ICON(obj)	G_TYPE_CHECK_INSTANCE_TYPE((obj), mn_mail_icon_get_type ())

#define MN_MAIL_ICON_GET_CLASS(obj)	G_TYPE_INSTANCE_GET_CLASS((obj), mn_mail_icon_get_type(), MNMailIconClass)

/* Private structure type */
typedef struct _MNMailIconPrivate MNMailIconPrivate;

/*
 * Main object structure
 */
#ifndef __TYPEDEF_MN_MAIL_ICON__
#define __TYPEDEF_MN_MAIL_ICON__
typedef struct _MNMailIcon MNMailIcon;
#endif
struct _MNMailIcon {
	EggTrayIcon __parent__;
	/*< public >*/
	GtkWidget * mail_reader_item;
	GtkWidget * open_latest_message_item;
	GtkWidget * update_item;
	GtkWidget * consider_new_mail_as_read_item;
	/*< private >*/
	MNMailIconPrivate *_priv;
};

/*
 * Class definition
 */
typedef struct _MNMailIconClass MNMailIconClass;
struct _MNMailIconClass {
	EggTrayIconClass __parent__;
	/*signal*/void (* activate) (MNMailIcon * self);
	/*signal*/void (* activate_mail_reader) (MNMailIcon * self);
	/*signal*/void (* activate_open_latest_message) (MNMailIcon * self);
	/*signal*/void (* activate_consider_new_mail_as_read) (MNMailIcon * self);
	/*signal*/void (* activate_update) (MNMailIcon * self);
	/*signal*/void (* activate_properties) (MNMailIcon * self);
	/*signal*/void (* activate_help) (MNMailIcon * self);
	/*signal*/void (* activate_about) (MNMailIcon * self);
	/*signal*/void (* activate_remove) (MNMailIcon * self);
};


/*
 * Public methods
 */
GType	mn_mail_icon_get_type	(void) G_GNUC_CONST;
#line 260 "src/mn-mail-icon.gob"
void 	mn_mail_icon_set_from_stock	(MNMailIcon * self,
					const char * stock_id);
#line 76 "mn-mail-icon.h"
#line 269 "src/mn-mail-icon.gob"
void 	mn_mail_icon_set_blinking	(MNMailIcon * self,
					gboolean blinking);
#line 80 "mn-mail-icon.h"
#line 275 "src/mn-mail-icon.gob"
void 	mn_mail_icon_set_count	(MNMailIcon * self,
					int count);
#line 84 "mn-mail-icon.h"
#line 281 "src/mn-mail-icon.gob"
void 	mn_mail_icon_set_tip	(MNMailIcon * self,
					const char * tip);
#line 88 "mn-mail-icon.h"
#line 287 "src/mn-mail-icon.gob"
void 	mn_mail_icon_set_tip_widget	(MNMailIcon * self,
					GtkWidget * tip_widget);
#line 92 "mn-mail-icon.h"
#line 293 "src/mn-mail-icon.gob"
GtkWidget * 	mn_mail_icon_new	(void);
#line 95 "mn-mail-icon.h"

/*
 * Signal connection wrapper macros
 */
#if defined(__GNUC__) && !defined(__STRICT_ANSI__)
#define mn_mail_icon_connect__activate(object,func,data)	g_signal_connect(MN_MAIL_ICON(__extension__ ({MNMailIcon *___object = (object); ___object; })),"activate",(GCallback) __extension__ ({void (* ___activate) (MNMailIcon * ___fake___self, gpointer ___data ) = (func); ___activate; }), (data))
#define mn_mail_icon_connect_after__activate(object,func,data)	g_signal_connect_after(MN_MAIL_ICON(__extension__ ({MNMailIcon *___object = (object); ___object; })),"activate",(GCallback) __extension__ ({void (* ___activate) (MNMailIcon * ___fake___self, gpointer ___data ) = (func); ___activate; }), (data))
#define mn_mail_icon_connect_data__activate(object,func,data,destroy_data,flags)	g_signal_connect_data(MN_MAIL_ICON(__extension__ ({MNMailIcon *___object = (object); ___object; })),"activate",(GCallback) __extension__ ({void (* ___activate) (MNMailIcon * ___fake___self, gpointer ___data ) = (func); ___activate; }), (data), (destroy_data), (GConnectFlags)(flags))
#define mn_mail_icon_connect__activate_mail_reader(object,func,data)	g_signal_connect(MN_MAIL_ICON(__extension__ ({MNMailIcon *___object = (object); ___object; })),"activate_mail_reader",(GCallback) __extension__ ({void (* ___activate_mail_reader) (MNMailIcon * ___fake___self, gpointer ___data ) = (func); ___activate_mail_reader; }), (data))
#define mn_mail_icon_connect_after__activate_mail_reader(object,func,data)	g_signal_connect_after(MN_MAIL_ICON(__extension__ ({MNMailIcon *___object = (object); ___object; })),"activate_mail_reader",(GCallback) __extension__ ({void (* ___activate_mail_reader) (MNMailIcon * ___fake___self, gpointer ___data ) = (func); ___activate_mail_reader; }), (data))
#define mn_mail_icon_connect_data__activate_mail_reader(object,func,data,destroy_data,flags)	g_signal_connect_data(MN_MAIL_ICON(__extension__ ({MNMailIcon *___object = (object); ___object; })),"activate_mail_reader",(GCallback) __extension__ ({void (* ___activate_mail_reader) (MNMailIcon * ___fake___self, gpointer ___data ) = (func); ___activate_mail_reader; }), (data), (destroy_data), (GConnectFlags)(flags))
#define mn_mail_icon_connect__activate_open_latest_message(object,func,data)	g_signal_connect(MN_MAIL_ICON(__extension__ ({MNMailIcon *___object = (object); ___object; })),"activate_open_latest_message",(GCallback) __extension__ ({void (* ___activate_open_latest_message) (MNMailIcon * ___fake___self, gpointer ___data ) = (func); ___activate_open_latest_message; }), (data))
#define mn_mail_icon_connect_after__activate_open_latest_message(object,func,data)	g_signal_connect_after(MN_MAIL_ICON(__extension__ ({MNMailIcon *___object = (object); ___object; })),"activate_open_latest_message",(GCallback) __extension__ ({void (* ___activate_open_latest_message) (MNMailIcon * ___fake___self, gpointer ___data ) = (func); ___activate_open_latest_message; }), (data))
#define mn_mail_icon_connect_data__activate_open_latest_message(object,func,data,destroy_data,flags)	g_signal_connect_data(MN_MAIL_ICON(__extension__ ({MNMailIcon *___object = (object); ___object; })),"activate_open_latest_message",(GCallback) __extension__ ({void (* ___activate_open_latest_message) (MNMailIcon * ___fake___self, gpointer ___data ) = (func); ___activate_open_latest_message; }), (data), (destroy_data), (GConnectFlags)(flags))
#define mn_mail_icon_connect__activate_consider_new_mail_as_read(object,func,data)	g_signal_connect(MN_MAIL_ICON(__extension__ ({MNMailIcon *___object = (object); ___object; })),"activate_consider_new_mail_as_read",(GCallback) __extension__ ({void (* ___activate_consider_new_mail_as_read) (MNMailIcon * ___fake___self, gpointer ___data ) = (func); ___activate_consider_new_mail_as_read; }), (data))
#define mn_mail_icon_connect_after__activate_consider_new_mail_as_read(object,func,data)	g_signal_connect_after(MN_MAIL_ICON(__extension__ ({MNMailIcon *___object = (object); ___object; })),"activate_consider_new_mail_as_read",(GCallback) __extension__ ({void (* ___activate_consider_new_mail_as_read) (MNMailIcon * ___fake___self, gpointer ___data ) = (func); ___activate_consider_new_mail_as_read; }), (data))
#define mn_mail_icon_connect_data__activate_consider_new_mail_as_read(object,func,data,destroy_data,flags)	g_signal_connect_data(MN_MAIL_ICON(__extension__ ({MNMailIcon *___object = (object); ___object; })),"activate_consider_new_mail_as_read",(GCallback) __extension__ ({void (* ___activate_consider_new_mail_as_read) (MNMailIcon * ___fake___self, gpointer ___data ) = (func); ___activate_consider_new_mail_as_read; }), (data), (destroy_data), (GConnectFlags)(flags))
#define mn_mail_icon_connect__activate_update(object,func,data)	g_signal_connect(MN_MAIL_ICON(__extension__ ({MNMailIcon *___object = (object); ___object; })),"activate_update",(GCallback) __extension__ ({void (* ___activate_update) (MNMailIcon * ___fake___self, gpointer ___data ) = (func); ___activate_update; }), (data))
#define mn_mail_icon_connect_after__activate_update(object,func,data)	g_signal_connect_after(MN_MAIL_ICON(__extension__ ({MNMailIcon *___object = (object); ___object; })),"activate_update",(GCallback) __extension__ ({void (* ___activate_update) (MNMailIcon * ___fake___self, gpointer ___data ) = (func); ___activate_update; }), (data))
#define mn_mail_icon_connect_data__activate_update(object,func,data,destroy_data,flags)	g_signal_connect_data(MN_MAIL_ICON(__extension__ ({MNMailIcon *___object = (object); ___object; })),"activate_update",(GCallback) __extension__ ({void (* ___activate_update) (MNMailIcon * ___fake___self, gpointer ___data ) = (func); ___activate_update; }), (data), (destroy_data), (GConnectFlags)(flags))
#define mn_mail_icon_connect__activate_properties(object,func,data)	g_signal_connect(MN_MAIL_ICON(__extension__ ({MNMailIcon *___object = (object); ___object; })),"activate_properties",(GCallback) __extension__ ({void (* ___activate_properties) (MNMailIcon * ___fake___self, gpointer ___data ) = (func); ___activate_properties; }), (data))
#define mn_mail_icon_connect_after__activate_properties(object,func,data)	g_signal_connect_after(MN_MAIL_ICON(__extension__ ({MNMailIcon *___object = (object); ___object; })),"activate_properties",(GCallback) __extension__ ({void (* ___activate_properties) (MNMailIcon * ___fake___self, gpointer ___data ) = (func); ___activate_properties; }), (data))
#define mn_mail_icon_connect_data__activate_properties(object,func,data,destroy_data,flags)	g_signal_connect_data(MN_MAIL_ICON(__extension__ ({MNMailIcon *___object = (object); ___object; })),"activate_properties",(GCallback) __extension__ ({void (* ___activate_properties) (MNMailIcon * ___fake___self, gpointer ___data ) = (func); ___activate_properties; }), (data), (destroy_data), (GConnectFlags)(flags))
#define mn_mail_icon_connect__activate_help(object,func,data)	g_signal_connect(MN_MAIL_ICON(__extension__ ({MNMailIcon *___object = (object); ___object; })),"activate_help",(GCallback) __extension__ ({void (* ___activate_help) (MNMailIcon * ___fake___self, gpointer ___data ) = (func); ___activate_help; }), (data))
#define mn_mail_icon_connect_after__activate_help(object,func,data)	g_signal_connect_after(MN_MAIL_ICON(__extension__ ({MNMailIcon *___object = (object); ___object; })),"activate_help",(GCallback) __extension__ ({void (* ___activate_help) (MNMailIcon * ___fake___self, gpointer ___data ) = (func); ___activate_help; }), (data))
#define mn_mail_icon_connect_data__activate_help(object,func,data,destroy_data,flags)	g_signal_connect_data(MN_MAIL_ICON(__extension__ ({MNMailIcon *___object = (object); ___object; })),"activate_help",(GCallback) __extension__ ({void (* ___activate_help) (MNMailIcon * ___fake___self, gpointer ___data ) = (func); ___activate_help; }), (data), (destroy_data), (GConnectFlags)(flags))
#define mn_mail_icon_connect__activate_about(object,func,data)	g_signal_connect(MN_MAIL_ICON(__extension__ ({MNMailIcon *___object = (object); ___object; })),"activate_about",(GCallback) __extension__ ({void (* ___activate_about) (MNMailIcon * ___fake___self, gpointer ___data ) = (func); ___activate_about; }), (data))
#define mn_mail_icon_connect_after__activate_about(object,func,data)	g_signal_connect_after(MN_MAIL_ICON(__extension__ ({MNMailIcon *___object = (object); ___object; })),"activate_about",(GCallback) __extension__ ({void (* ___activate_about) (MNMailIcon * ___fake___self, gpointer ___data ) = (func); ___activate_about; }), (data))
#define mn_mail_icon_connect_data__activate_about(object,func,data,destroy_data,flags)	g_signal_connect_data(MN_MAIL_ICON(__extension__ ({MNMailIcon *___object = (object); ___object; })),"activate_about",(GCallback) __extension__ ({void (* ___activate_about) (MNMailIcon * ___fake___self, gpointer ___data ) = (func); ___activate_about; }), (data), (destroy_data), (GConnectFlags)(flags))
#define mn_mail_icon_connect__activate_remove(object,func,data)	g_signal_connect(MN_MAIL_ICON(__extension__ ({MNMailIcon *___object = (object); ___object; })),"activate_remove",(GCallback) __extension__ ({void (* ___activate_remove) (MNMailIcon * ___fake___self, gpointer ___data ) = (func); ___activate_remove; }), (data))
#define mn_mail_icon_connect_after__activate_remove(object,func,data)	g_signal_connect_after(MN_MAIL_ICON(__extension__ ({MNMailIcon *___object = (object); ___object; })),"activate_remove",(GCallback) __extension__ ({void (* ___activate_remove) (MNMailIcon * ___fake___self, gpointer ___data ) = (func); ___activate_remove; }), (data))
#define mn_mail_icon_connect_data__activate_remove(object,func,data,destroy_data,flags)	g_signal_connect_data(MN_MAIL_ICON(__extension__ ({MNMailIcon *___object = (object); ___object; })),"activate_remove",(GCallback) __extension__ ({void (* ___activate_remove) (MNMailIcon * ___fake___self, gpointer ___data ) = (func); ___activate_remove; }), (data), (destroy_data), (GConnectFlags)(flags))
#else /* __GNUC__ && !__STRICT_ANSI__ */
#define mn_mail_icon_connect__activate(object,func,data)	g_signal_connect(MN_MAIL_ICON(object),"activate",(GCallback)(func),(data))
#define mn_mail_icon_connect_after__activate(object,func,data)	g_signal_connect_after(MN_MAIL_ICON(object),"activate",(GCallback)(func),(data))
#define mn_mail_icon_connect_data__activate(object,func,data,destroy_data,flags)	g_signal_connect_data(MN_MAIL_ICON(object),"activate",(GCallback)(func),(data),(destroy_data),(GConnectFlags)(flags))
#define mn_mail_icon_connect__activate_mail_reader(object,func,data)	g_signal_connect(MN_MAIL_ICON(object),"activate_mail_reader",(GCallback)(func),(data))
#define mn_mail_icon_connect_after__activate_mail_reader(object,func,data)	g_signal_connect_after(MN_MAIL_ICON(object),"activate_mail_reader",(GCallback)(func),(data))
#define mn_mail_icon_connect_data__activate_mail_reader(object,func,data,destroy_data,flags)	g_signal_connect_data(MN_MAIL_ICON(object),"activate_mail_reader",(GCallback)(func),(data),(destroy_data),(GConnectFlags)(flags))
#define mn_mail_icon_connect__activate_open_latest_message(object,func,data)	g_signal_connect(MN_MAIL_ICON(object),"activate_open_latest_message",(GCallback)(func),(data))
#define mn_mail_icon_connect_after__activate_open_latest_message(object,func,data)	g_signal_connect_after(MN_MAIL_ICON(object),"activate_open_latest_message",(GCallback)(func),(data))
#define mn_mail_icon_connect_data__activate_open_latest_message(object,func,data,destroy_data,flags)	g_signal_connect_data(MN_MAIL_ICON(object),"activate_open_latest_message",(GCallback)(func),(data),(destroy_data),(GConnectFlags)(flags))
#define mn_mail_icon_connect__activate_consider_new_mail_as_read(object,func,data)	g_signal_connect(MN_MAIL_ICON(object),"activate_consider_new_mail_as_read",(GCallback)(func),(data))
#define mn_mail_icon_connect_after__activate_consider_new_mail_as_read(object,func,data)	g_signal_connect_after(MN_MAIL_ICON(object),"activate_consider_new_mail_as_read",(GCallback)(func),(data))
#define mn_mail_icon_connect_data__activate_consider_new_mail_as_read(object,func,data,destroy_data,flags)	g_signal_connect_data(MN_MAIL_ICON(object),"activate_consider_new_mail_as_read",(GCallback)(func),(data),(destroy_data),(GConnectFlags)(flags))
#define mn_mail_icon_connect__activate_update(object,func,data)	g_signal_connect(MN_MAIL_ICON(object),"activate_update",(GCallback)(func),(data))
#define mn_mail_icon_connect_after__activate_update(object,func,data)	g_signal_connect_after(MN_MAIL_ICON(object),"activate_update",(GCallback)(func),(data))
#define mn_mail_icon_connect_data__activate_update(object,func,data,destroy_data,flags)	g_signal_connect_data(MN_MAIL_ICON(object),"activate_update",(GCallback)(func),(data),(destroy_data),(GConnectFlags)(flags))
#define mn_mail_icon_connect__activate_properties(object,func,data)	g_signal_connect(MN_MAIL_ICON(object),"activate_properties",(GCallback)(func),(data))
#define mn_mail_icon_connect_after__activate_properties(object,func,data)	g_signal_connect_after(MN_MAIL_ICON(object),"activate_properties",(GCallback)(func),(data))
#define mn_mail_icon_connect_data__activate_properties(object,func,data,destroy_data,flags)	g_signal_connect_data(MN_MAIL_ICON(object),"activate_properties",(GCallback)(func),(data),(destroy_data),(GConnectFlags)(flags))
#define mn_mail_icon_connect__activate_help(object,func,data)	g_signal_connect(MN_MAIL_ICON(object),"activate_help",(GCallback)(func),(data))
#define mn_mail_icon_connect_after__activate_help(object,func,data)	g_signal_connect_after(MN_MAIL_ICON(object),"activate_help",(GCallback)(func),(data))
#define mn_mail_icon_connect_data__activate_help(object,func,data,destroy_data,flags)	g_signal_connect_data(MN_MAIL_ICON(object),"activate_help",(GCallback)(func),(data),(destroy_data),(GConnectFlags)(flags))
#define mn_mail_icon_connect__activate_about(object,func,data)	g_signal_connect(MN_MAIL_ICON(object),"activate_about",(GCallback)(func),(data))
#define mn_mail_icon_connect_after__activate_about(object,func,data)	g_signal_connect_after(MN_MAIL_ICON(object),"activate_about",(GCallback)(func),(data))
#define mn_mail_icon_connect_data__activate_about(object,func,data,destroy_data,flags)	g_signal_connect_data(MN_MAIL_ICON(object),"activate_about",(GCallback)(func),(data),(destroy_data),(GConnectFlags)(flags))
#define mn_mail_icon_connect__activate_remove(object,func,data)	g_signal_connect(MN_MAIL_ICON(object),"activate_remove",(GCallback)(func),(data))
#define mn_mail_icon_connect_after__activate_remove(object,func,data)	g_signal_connect_after(MN_MAIL_ICON(object),"activate_remove",(GCallback)(func),(data))
#define mn_mail_icon_connect_data__activate_remove(object,func,data,destroy_data,flags)	g_signal_connect_data(MN_MAIL_ICON(object),"activate_remove",(GCallback)(func),(data),(destroy_data),(GConnectFlags)(flags))
#endif /* __GNUC__ && !__STRICT_ANSI__ */


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
