/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_BASE_MBOX_MAILBOX_BACKEND_PRIVATE_H__
#define __MN_BASE_MBOX_MAILBOX_BACKEND_PRIVATE_H__

#include "mn-base-mbox-mailbox-backend.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

struct _MNBaseMboxMailboxBackendPrivate {
#line 47 "src/mn-base-mbox-mailbox-backend.gob"
	GnomeVFSURI * uri;
#line 48 "src/mn-base-mbox-mailbox-backend.gob"
	GnomeVFSFileSize last_size;
#line 49 "src/mn-base-mbox-mailbox-backend.gob"
	time_t last_mtime;
#line 50 "src/mn-base-mbox-mailbox-backend.gob"
	time_t last_ctime;
#line 52 "src/mn-base-mbox-mailbox-backend.gob"
	GMutex * mutex;
#line 24 "mn-base-mbox-mailbox-backend-private.h"
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
