/* Generated by GOB (v2.0.15)   (do not edit directly) */

#include <glib.h>
#include <glib-object.h>


#include "mn-message.h"

#ifndef __MN_EVOLUTION_MESSAGE_H__
#define __MN_EVOLUTION_MESSAGE_H__

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/*
 * Type checking and casting macros
 */
#define MN_TYPE_EVOLUTION_MESSAGE	(mn_evolution_message_get_type())
#define MN_EVOLUTION_MESSAGE(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), mn_evolution_message_get_type(), MNEvolutionMessage)
#define MN_EVOLUTION_MESSAGE_CONST(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), mn_evolution_message_get_type(), MNEvolutionMessage const)
#define MN_EVOLUTION_MESSAGE_CLASS(klass)	G_TYPE_CHECK_CLASS_CAST((klass), mn_evolution_message_get_type(), MNEvolutionMessageClass)
#define MN_IS_EVOLUTION_MESSAGE(obj)	G_TYPE_CHECK_INSTANCE_TYPE((obj), mn_evolution_message_get_type ())

#define MN_EVOLUTION_MESSAGE_GET_CLASS(obj)	G_TYPE_INSTANCE_GET_CLASS((obj), mn_evolution_message_get_type(), MNEvolutionMessageClass)

/* Private structure type */
typedef struct _MNEvolutionMessagePrivate MNEvolutionMessagePrivate;

/*
 * Main object structure
 */
#ifndef __TYPEDEF_MN_EVOLUTION_MESSAGE__
#define __TYPEDEF_MN_EVOLUTION_MESSAGE__
typedef struct _MNEvolutionMessage MNEvolutionMessage;
#endif
struct _MNEvolutionMessage {
	MNMessage __parent__;
	/*< private >*/
	MNEvolutionMessagePrivate *_priv;
};

/*
 * Class definition
 */
typedef struct _MNEvolutionMessageClass MNEvolutionMessageClass;
struct _MNEvolutionMessageClass {
	MNMessageClass __parent__;
};


/*
 * Public methods
 */
GType	mn_evolution_message_get_type	(void) G_GNUC_CONST;
#line 64 "src/mn-evolution-message.gob"
MNMessage * 	mn_evolution_message_new	(MNMailbox * mailbox,
					time_t sent_time,
					const char * id,
					const char * from,
					const char * subject,
					MNMessageFlags flags,
					const char * uid);
#line 66 "mn-evolution-message.h"

/*
 * Argument wrapping macros
 */
#if defined(__GNUC__) && !defined(__STRICT_ANSI__)
#define MN_EVOLUTION_MESSAGE_PROP_UID(arg)    	"uid", __extension__ ({gchar *z = (arg); z;})
#define MN_EVOLUTION_MESSAGE_GET_PROP_UID(arg)	"uid", __extension__ ({gchar **z = (arg); z;})
#else /* __GNUC__ && !__STRICT_ANSI__ */
#define MN_EVOLUTION_MESSAGE_PROP_UID(arg)    	"uid",(gchar *)(arg)
#define MN_EVOLUTION_MESSAGE_GET_PROP_UID(arg)	"uid",(gchar **)(arg)
#endif /* __GNUC__ && !__STRICT_ANSI__ */


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
