/* Generated by GOB (v2.0.15)   (do not edit directly) */

#include <glib.h>
#include <glib-object.h>


#include <dbus/dbus-glib.h>

#ifndef __MN_EVOLUTION_SERVER_H__
#define __MN_EVOLUTION_SERVER_H__

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/*
 * Type checking and casting macros
 */
#define MN_TYPE_EVOLUTION_SERVER	(mn_evolution_server_get_type())
#define MN_EVOLUTION_SERVER(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), mn_evolution_server_get_type(), MNEvolutionServer)
#define MN_EVOLUTION_SERVER_CONST(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), mn_evolution_server_get_type(), MNEvolutionServer const)
#define MN_EVOLUTION_SERVER_CLASS(klass)	G_TYPE_CHECK_CLASS_CAST((klass), mn_evolution_server_get_type(), MNEvolutionServerClass)
#define MN_IS_EVOLUTION_SERVER(obj)	G_TYPE_CHECK_INSTANCE_TYPE((obj), mn_evolution_server_get_type ())

#define MN_EVOLUTION_SERVER_GET_CLASS(obj)	G_TYPE_INSTANCE_GET_CLASS((obj), mn_evolution_server_get_type(), MNEvolutionServerClass)

/*
 * Main object structure
 */
#ifndef __TYPEDEF_MN_EVOLUTION_SERVER__
#define __TYPEDEF_MN_EVOLUTION_SERVER__
typedef struct _MNEvolutionServer MNEvolutionServer;
#endif
struct _MNEvolutionServer {
	GObject __parent__;
};

/*
 * Class definition
 */
typedef struct _MNEvolutionServerClass MNEvolutionServerClass;
struct _MNEvolutionServerClass {
	GObjectClass __parent__;
	/*signal*/void (* folder_changed) (MNEvolutionServer * self, const char * uri);
	/*signal*/void (* message_reading) (MNEvolutionServer * self, const char * uri);
};


/*
 * Public methods
 */
GType	mn_evolution_server_get_type	(void) G_GNUC_CONST;
#line 69 "src/mn-evolution-server.gob"
void 	mn_evolution_server_folder_changed	(MNEvolutionServer * self,
					const char * uri);
#line 58 "mn-evolution-server.h"
#line 72 "src/mn-evolution-server.gob"
void 	mn_evolution_server_message_reading	(MNEvolutionServer * self,
					const char * uri);
#line 62 "mn-evolution-server.h"
#line 374 "src/mn-evolution-server.gob"
MNEvolutionServer * 	mn_evolution_server_new	(void);
#line 65 "mn-evolution-server.h"

/*
 * Signal connection wrapper macros
 */
#if defined(__GNUC__) && !defined(__STRICT_ANSI__)
#define mn_evolution_server_connect__folder_changed(object,func,data)	g_signal_connect(MN_EVOLUTION_SERVER(__extension__ ({MNEvolutionServer *___object = (object); ___object; })),"folder_changed",(GCallback) __extension__ ({void (* ___folder_changed) (MNEvolutionServer * ___fake___self, const char * ___fake___uri, gpointer ___data ) = (func); ___folder_changed; }), (data))
#define mn_evolution_server_connect_after__folder_changed(object,func,data)	g_signal_connect_after(MN_EVOLUTION_SERVER(__extension__ ({MNEvolutionServer *___object = (object); ___object; })),"folder_changed",(GCallback) __extension__ ({void (* ___folder_changed) (MNEvolutionServer * ___fake___self, const char * ___fake___uri, gpointer ___data ) = (func); ___folder_changed; }), (data))
#define mn_evolution_server_connect_data__folder_changed(object,func,data,destroy_data,flags)	g_signal_connect_data(MN_EVOLUTION_SERVER(__extension__ ({MNEvolutionServer *___object = (object); ___object; })),"folder_changed",(GCallback) __extension__ ({void (* ___folder_changed) (MNEvolutionServer * ___fake___self, const char * ___fake___uri, gpointer ___data ) = (func); ___folder_changed; }), (data), (destroy_data), (GConnectFlags)(flags))
#define mn_evolution_server_connect__message_reading(object,func,data)	g_signal_connect(MN_EVOLUTION_SERVER(__extension__ ({MNEvolutionServer *___object = (object); ___object; })),"message_reading",(GCallback) __extension__ ({void (* ___message_reading) (MNEvolutionServer * ___fake___self, const char * ___fake___uri, gpointer ___data ) = (func); ___message_reading; }), (data))
#define mn_evolution_server_connect_after__message_reading(object,func,data)	g_signal_connect_after(MN_EVOLUTION_SERVER(__extension__ ({MNEvolutionServer *___object = (object); ___object; })),"message_reading",(GCallback) __extension__ ({void (* ___message_reading) (MNEvolutionServer * ___fake___self, const char * ___fake___uri, gpointer ___data ) = (func); ___message_reading; }), (data))
#define mn_evolution_server_connect_data__message_reading(object,func,data,destroy_data,flags)	g_signal_connect_data(MN_EVOLUTION_SERVER(__extension__ ({MNEvolutionServer *___object = (object); ___object; })),"message_reading",(GCallback) __extension__ ({void (* ___message_reading) (MNEvolutionServer * ___fake___self, const char * ___fake___uri, gpointer ___data ) = (func); ___message_reading; }), (data), (destroy_data), (GConnectFlags)(flags))
#else /* __GNUC__ && !__STRICT_ANSI__ */
#define mn_evolution_server_connect__folder_changed(object,func,data)	g_signal_connect(MN_EVOLUTION_SERVER(object),"folder_changed",(GCallback)(func),(data))
#define mn_evolution_server_connect_after__folder_changed(object,func,data)	g_signal_connect_after(MN_EVOLUTION_SERVER(object),"folder_changed",(GCallback)(func),(data))
#define mn_evolution_server_connect_data__folder_changed(object,func,data,destroy_data,flags)	g_signal_connect_data(MN_EVOLUTION_SERVER(object),"folder_changed",(GCallback)(func),(data),(destroy_data),(GConnectFlags)(flags))
#define mn_evolution_server_connect__message_reading(object,func,data)	g_signal_connect(MN_EVOLUTION_SERVER(object),"message_reading",(GCallback)(func),(data))
#define mn_evolution_server_connect_after__message_reading(object,func,data)	g_signal_connect_after(MN_EVOLUTION_SERVER(object),"message_reading",(GCallback)(func),(data))
#define mn_evolution_server_connect_data__message_reading(object,func,data,destroy_data,flags)	g_signal_connect_data(MN_EVOLUTION_SERVER(object),"message_reading",(GCallback)(func),(data),(destroy_data),(GConnectFlags)(flags))
#endif /* __GNUC__ && !__STRICT_ANSI__ */


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
