/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_WEBMAIL_MAILBOX_PRIVATE_H__
#define __MN_WEBMAIL_MAILBOX_PRIVATE_H__

#include "mn-webmail-mailbox.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#line 65 "src/mn-webmail-mailbox.gob"
void 	mn_webmail_mailbox_parse_username	(MNWebmailMailbox * self, char ** username, char ** domain);
#line 15 "mn-webmail-mailbox-private.h"
#line 101 "src/mn-webmail-mailbox.gob"
char * 	mn_webmail_mailbox_create_spool_file	(GError ** err);
#line 18 "mn-webmail-mailbox-private.h"
#line 116 "src/mn-webmail-mailbox.gob"
char * 	mn_webmail_mailbox_write_temporary_file	(const char * contents, const char * encoding, GError ** err);
#line 21 "mn-webmail-mailbox-private.h"
#line 189 "src/mn-webmail-mailbox.gob"
void 	mn_webmail_mailbox_print_config	(MNWebmailMailbox * self, const char * config);
#line 24 "mn-webmail-mailbox-private.h"
#line 208 "src/mn-webmail-mailbox.gob"
gboolean 	mn_webmail_mailbox_run_helper	(MNWebmailMailbox * self, const char * name, const char * command, GError ** err);
#line 27 "mn-webmail-mailbox-private.h"
#line 265 "src/mn-webmail-mailbox.gob"
gboolean 	mn_webmail_mailbox_read_spool_file	(MNWebmailMailbox * self, const char * spool_file, GError ** err);
#line 30 "mn-webmail-mailbox-private.h"

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
