/* Generated by GOB (v2.0.15)   (do not edit directly) */

/* End world hunger, donate to the World Food Programme, http://www.wfp.org */

#define GOB_VERSION_MAJOR 2
#define GOB_VERSION_MINOR 0
#define GOB_VERSION_PATCHLEVEL 15

#define selfp (self->_priv)

#include <string.h> /* memset() */

#include "mn-yahoo-mailbox-properties.h"

#include "mn-yahoo-mailbox-properties-private.h"

#ifdef G_LIKELY
#define ___GOB_LIKELY(expr) G_LIKELY(expr)
#define ___GOB_UNLIKELY(expr) G_UNLIKELY(expr)
#else /* ! G_LIKELY */
#define ___GOB_LIKELY(expr) (expr)
#define ___GOB_UNLIKELY(expr) (expr)
#endif /* G_LIKELY */

#line 24 "src/mn-yahoo-mailbox-properties.gob"

#include "mn-stock.h"

#line 30 "mn-yahoo-mailbox-properties.c"
/* self casting macros */
#define SELF(x) MN_YAHOO_MAILBOX_PROPERTIES(x)
#define SELF_CONST(x) MN_YAHOO_MAILBOX_PROPERTIES_CONST(x)
#define IS_SELF(x) MN_IS_YAHOO_MAILBOX_PROPERTIES(x)
#define TYPE_SELF MN_TYPE_YAHOO_MAILBOX_PROPERTIES
#define SELF_CLASS(x) MN_YAHOO_MAILBOX_PROPERTIES_CLASS(x)

#define SELF_GET_CLASS(x) MN_YAHOO_MAILBOX_PROPERTIES_GET_CLASS(x)

/* self typedefs */
typedef MNYahooMailboxProperties Self;
typedef MNYahooMailboxPropertiesClass SelfClass;

/* here are local prototypes */
#line 0 "src/mn-yahoo-mailbox-properties.gob"
static void mn_yahoo_mailbox_properties_init (MNYahooMailboxProperties * o);
#line 47 "mn-yahoo-mailbox-properties.c"
#line 30 "src/mn-yahoo-mailbox-properties.gob"
static void mn_yahoo_mailbox_properties_class_init (MNYahooMailboxPropertiesClass * class);
#line 50 "mn-yahoo-mailbox-properties.c"

/* pointer to the class of our parent */
static MNWebmailMailboxPropertiesClass *parent_class = NULL;

GType
mn_yahoo_mailbox_properties_get_type (void)
{
	static GType type = 0;

	if ___GOB_UNLIKELY(type == 0) {
		static const GTypeInfo info = {
			sizeof (MNYahooMailboxPropertiesClass),
			(GBaseInitFunc) NULL,
			(GBaseFinalizeFunc) NULL,
			(GClassInitFunc) mn_yahoo_mailbox_properties_class_init,
			(GClassFinalizeFunc) NULL,
			NULL /* class_data */,
			sizeof (MNYahooMailboxProperties),
			0 /* n_preallocs */,
			(GInstanceInitFunc) mn_yahoo_mailbox_properties_init,
			NULL
		};

		type = g_type_register_static (MN_TYPE_WEBMAIL_MAILBOX_PROPERTIES, "MNYahooMailboxProperties", &info, (GTypeFlags)0);
	}

	return type;
}

/* a macro for creating a new object of our type */
#define GET_NEW ((MNYahooMailboxProperties *)g_object_new(mn_yahoo_mailbox_properties_get_type(), NULL))

/* a function for creating a new object of our type */
#include <stdarg.h>
static MNYahooMailboxProperties * GET_NEW_VARG (const char *first, ...) G_GNUC_UNUSED;
static MNYahooMailboxProperties *
GET_NEW_VARG (const char *first, ...)
{
	MNYahooMailboxProperties *ret;
	va_list ap;
	va_start (ap, first);
	ret = (MNYahooMailboxProperties *)g_object_new_valist (mn_yahoo_mailbox_properties_get_type (), first, ap);
	va_end (ap);
	return ret;
}

static void 
mn_yahoo_mailbox_properties_init (MNYahooMailboxProperties * o G_GNUC_UNUSED)
{
#define __GOB_FUNCTION__ "MN:Yahoo:Mailbox:Properties::init"
}
#undef __GOB_FUNCTION__
#line 30 "src/mn-yahoo-mailbox-properties.gob"
static void 
mn_yahoo_mailbox_properties_class_init (MNYahooMailboxPropertiesClass * class G_GNUC_UNUSED)
{
#line 107 "mn-yahoo-mailbox-properties.c"
#define __GOB_FUNCTION__ "MN:Yahoo:Mailbox:Properties::class_init"

	parent_class = g_type_class_ref (MN_TYPE_WEBMAIL_MAILBOX_PROPERTIES);

 {
#line 31 "src/mn-yahoo-mailbox-properties.gob"

    MNMailboxPropertiesClass *p_class = MN_MAILBOX_PROPERTIES_CLASS(class);

    p_class->type = "yahoo";
    p_class->stock_id = MN_STOCK_YAHOO;
    p_class->combo_label = "Yahoo! Mail";
  
#line 121 "mn-yahoo-mailbox-properties.c"
 }
}
#undef __GOB_FUNCTION__


