/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_EVOLUTION_FOLDER_TREE_SERVER_PRIVATE_H__
#define __MN_EVOLUTION_FOLDER_TREE_SERVER_PRIVATE_H__

#include "mn-evolution-folder-tree-server.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#line 20 "src/mn-evolution-folder-tree-server.gob"

#include <gtk/gtk.h>

#line 18 "mn-evolution-folder-tree-server-private.h"
struct _MNEvolutionFolderTreeServerPrivate {
#line 37 "src/mn-evolution-folder-tree-server.gob"
	guint32 id;
#line 40 "src/mn-evolution-folder-tree-server.gob"
	GtkWidget * plug;
#line 41 "src/mn-evolution-folder-tree-server.gob"
	GtkWidget * tree;
#line 26 "mn-evolution-folder-tree-server-private.h"
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
