/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_PI_MAILBOX_PROPERTIES_PRIVATE_H__
#define __MN_PI_MAILBOX_PROPERTIES_PRIVATE_H__

#include "mn-pi-mailbox-properties.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#line 265 "src/mn-pi-mailbox-properties.gob"
void 	mn_pi_mailbox_properties_get_contents	(MNPIMailboxProperties * self, MNPIMailboxConnectionType * connection_type, char ** authmech, const char ** server, int * port);
#line 15 "mn-pi-mailbox-properties-private.h"
#line 300 "src/mn-pi-mailbox-properties.gob"
gboolean 	mn_pi_mailbox_properties_is_complete	(MNPIMailboxProperties * self);
#line 18 "mn-pi-mailbox-properties-private.h"

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
