/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_MAILBOX_PRIVATE_H__
#define __MN_MAILBOX_PRIVATE_H__

#include "mn-mailbox.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#line 34 "src/mn-mailbox.gob"

#include "mn-xml.h"

typedef enum
{
  /* load from mailboxes.xml */
  MN_MAILBOX_PARAM_LOAD		= MN_XML_PARAM_IMPORT,

  /* save to mailboxes.xml */
  MN_MAILBOX_PARAM_SAVE		= MN_XML_PARAM_EXPORT,

  /* load from and save to mailboxes.xml */
  MN_MAILBOX_PARAM_LOAD_SAVE	= MN_MAILBOX_PARAM_LOAD | MN_MAILBOX_PARAM_SAVE,

  /* ignore the case of a string property when checking if it has the default value */
  MN_MAILBOX_PARAM_IGNORE_CASE	= MN_XML_PARAM_IGNORE_CASE,

  /* require a string property to be non-empty */
  MN_MAILBOX_PARAM_REQUIRED	= 1 << (MN_XML_PARAM_USER_SHIFT + 0),
} MNMailboxParamFlags;

#line 36 "mn-mailbox-private.h"
struct _MNMailboxPrivate {
#line 133 "src/mn-mailbox.gob"
	gboolean _active;
#line 215 "src/mn-mailbox.gob"
	unsigned int check_timeout_id;
#line 224 "src/mn-mailbox.gob"
	gboolean poll;
#line 271 "src/mn-mailbox.gob"
	gboolean all_messages_set;
#line 274 "src/mn-mailbox.gob"
	GHashTable * all_messages;
#line 277 "src/mn-mailbox.gob"
	GHashTable * all_messages_by_mid;
#line 1126 "src/mn-mailbox.gob"
	gboolean checking_enabled;
#line 52 "mn-mailbox-private.h"
};
#line 291 "src/mn-mailbox.gob"
void 	mn_mailbox_set_messages	(MNMailbox * self, GSList * messages);
#line 56 "mn-mailbox-private.h"
#line 510 "src/mn-mailbox.gob"
void 	mn_mailbox_set_error	(MNMailbox * self, const char * format, ...) G_GNUC_PRINTF(2, 3);
#line 59 "mn-mailbox-private.h"
#line 1128 "src/mn-mailbox.gob"
void 	mn_mailbox_enable_checking	(MNMailbox * self);
#line 62 "mn-mailbox-private.h"
#line 1139 "src/mn-mailbox.gob"
void 	mn_mailbox_notice	(MNMailbox * self, const char * format, ...) G_GNUC_PRINTF(2, 3);
#line 65 "mn-mailbox-private.h"
#line 1157 "src/mn-mailbox.gob"
void 	mn_mailbox_warning	(MNMailbox * self, const char * format, ...) G_GNUC_PRINTF(2, 3);
#line 68 "mn-mailbox-private.h"
#line 1174 "src/mn-mailbox.gob"
MNMessage * 	mn_mailbox_get_message_from_mid	(MNMailbox * self, const char * mid);
#line 71 "mn-mailbox-private.h"

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
