/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_EVOLUTION_MESSAGE_PRIVATE_H__
#define __MN_EVOLUTION_MESSAGE_PRIVATE_H__

#include "mn-evolution-message.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

struct _MNEvolutionMessagePrivate {
#line 61 "src/mn-evolution-message.gob"
	char * uid;
#line 16 "mn-evolution-message-private.h"
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
