/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_REENTRANT_MAILBOX_PRIVATE_H__
#define __MN_REENTRANT_MAILBOX_PRIVATE_H__

#include "mn-reentrant-mailbox.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

struct _MNReentrantMailboxPrivate {
#line 38 "src/mn-reentrant-mailbox.gob"
	unsigned int queue_check_id;
#line 40 "src/mn-reentrant-mailbox.gob"
	int check_id;
#line 41 "src/mn-reentrant-mailbox.gob"
	int check_unique_id;
#line 20 "mn-reentrant-mailbox-private.h"
};
#line 69 "src/mn-reentrant-mailbox.gob"
void 	mn_reentrant_mailbox_queue_check	(MNReentrantMailbox * self);
#line 24 "mn-reentrant-mailbox-private.h"
#line 131 "src/mn-reentrant-mailbox.gob"
gboolean 	mn_reentrant_mailbox_check_aborted	(MNReentrantMailbox * self, int check_id);
#line 27 "mn-reentrant-mailbox-private.h"

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
