/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_SYLPHEED_MAILBOX_BACKEND_PRIVATE_H__
#define __MN_SYLPHEED_MAILBOX_BACKEND_PRIVATE_H__

#include "mn-sylpheed-mailbox-backend.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#line 24 "src/mn-sylpheed-mailbox-backend.gob"

#include "mn-locked-callback.h"

/* taken from procmsg.h in the Sylpheed sources */
#define SYLPHEED_MSG_NEW	(1U << 0)
#define SYLPHEED_MSG_UNREAD	(1U << 1)

/* taken from defs.h in the Sylpheed sources */
#define SYLPHEED_MARK_FILE	".sylpheed_mark"

typedef struct
{
  char	*position;
  gsize	bytes_left;
} ByteStream;

#line 31 "mn-sylpheed-mailbox-backend-private.h"
#line 448 "src/mn-sylpheed-mailbox-backend.gob"
GHashTable * 	mn_sylpheed_mailbox_backend_read_marks	(GnomeVFSURI * mailbox_uri, GError ** err);
#line 34 "mn-sylpheed-mailbox-backend-private.h"
#line 507 "src/mn-sylpheed-mailbox-backend.gob"
gboolean 	mn_sylpheed_mailbox_backend_write_marks	(GnomeVFSURI * mailbox_uri, GHashTable * marks, GError ** err);
#line 37 "mn-sylpheed-mailbox-backend-private.h"

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
