/* Generated by GOB (v2.0.15)   (do not edit directly) */

#include <glib.h>
#include <glib-object.h>


#include "mn-mailbox.h"

#ifndef __MN_TEST_MAILBOX_H__
#define __MN_TEST_MAILBOX_H__

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/*
 * Type checking and casting macros
 */
#define MN_TYPE_TEST_MAILBOX	(mn_test_mailbox_get_type())
#define MN_TEST_MAILBOX(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), mn_test_mailbox_get_type(), MNTestMailbox)
#define MN_TEST_MAILBOX_CONST(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), mn_test_mailbox_get_type(), MNTestMailbox const)
#define MN_TEST_MAILBOX_CLASS(klass)	G_TYPE_CHECK_CLASS_CAST((klass), mn_test_mailbox_get_type(), MNTestMailboxClass)
#define MN_IS_TEST_MAILBOX(obj)	G_TYPE_CHECK_INSTANCE_TYPE((obj), mn_test_mailbox_get_type ())

#define MN_TEST_MAILBOX_GET_CLASS(obj)	G_TYPE_INSTANCE_GET_CLASS((obj), mn_test_mailbox_get_type(), MNTestMailboxClass)

/*
 * Main object structure
 */
#ifndef __TYPEDEF_MN_TEST_MAILBOX__
#define __TYPEDEF_MN_TEST_MAILBOX__
typedef struct _MNTestMailbox MNTestMailbox;
#endif
struct _MNTestMailbox {
	MNMailbox __parent__;
};

/*
 * Class definition
 */
typedef struct _MNTestMailboxClass MNTestMailboxClass;
struct _MNTestMailboxClass {
	MNMailboxClass __parent__;
};


/*
 * Public methods
 */
GType	mn_test_mailbox_get_type	(void) G_GNUC_CONST;
#line 76 "src/mn-test-mailbox.gob"
MNMailbox * 	mn_test_mailbox_new	(void);
#line 55 "mn-test-mailbox.h"

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
