/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_POP3_MAILBOX_PRIVATE_H__
#define __MN_POP3_MAILBOX_PRIVATE_H__

#include "mn-pop3-mailbox.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

struct _MNPOP3MailboxPrivate {
#line 138 "src/mn-pop3-mailbox.gob"
	int login_delay;
#line 139 "src/mn-pop3-mailbox.gob"
	GTimer * authentication_timer;
#line 18 "mn-pop3-mailbox-private.h"
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
