/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_EVOLUTION_MAILBOX_PRIVATE_H__
#define __MN_EVOLUTION_MAILBOX_PRIVATE_H__

#include "mn-evolution-mailbox.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#line 24 "src/mn-evolution-mailbox.gob"

#include <time.h>
#include "mn-evolution-client.h"

#line 19 "mn-evolution-mailbox-private.h"
struct _MNEvolutionMailboxPrivate {
#line 65 "src/mn-evolution-mailbox.gob"
	MNEvolutionClient * client;
#line 66 "src/mn-evolution-mailbox.gob"
	gboolean signals_connected;
#line 67 "src/mn-evolution-mailbox.gob"
	DBusGProxyCall * pending_call;
#line 69 "src/mn-evolution-mailbox.gob"
	time_t last_browsed;
#line 29 "mn-evolution-mailbox-private.h"
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
