/* Generated by GOB (v2.0.15)   (do not edit directly) */

#include <glib.h>
#include <glib-object.h>


#include "mn-authenticated-mailbox.h"

#ifndef __MN_GMAIL_MAILBOX_H__
#define __MN_GMAIL_MAILBOX_H__

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/*
 * Type checking and casting macros
 */
#define MN_TYPE_GMAIL_MAILBOX	(mn_gmail_mailbox_get_type())
#define MN_GMAIL_MAILBOX(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), mn_gmail_mailbox_get_type(), MNGmailMailbox)
#define MN_GMAIL_MAILBOX_CONST(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), mn_gmail_mailbox_get_type(), MNGmailMailbox const)
#define MN_GMAIL_MAILBOX_CLASS(klass)	G_TYPE_CHECK_CLASS_CAST((klass), mn_gmail_mailbox_get_type(), MNGmailMailboxClass)
#define MN_IS_GMAIL_MAILBOX(obj)	G_TYPE_CHECK_INSTANCE_TYPE((obj), mn_gmail_mailbox_get_type ())

#define MN_GMAIL_MAILBOX_GET_CLASS(obj)	G_TYPE_INSTANCE_GET_CLASS((obj), mn_gmail_mailbox_get_type(), MNGmailMailboxClass)

/* Private structure type */
typedef struct _MNGmailMailboxPrivate MNGmailMailboxPrivate;

/*
 * Main object structure
 */
#ifndef __TYPEDEF_MN_GMAIL_MAILBOX__
#define __TYPEDEF_MN_GMAIL_MAILBOX__
typedef struct _MNGmailMailbox MNGmailMailbox;
#endif
struct _MNGmailMailbox {
	MNAuthenticatedMailbox __parent__;
	/*< public >*/
	char * location;
	char * label;
	/*< private >*/
	MNGmailMailboxPrivate *_priv;
};

/*
 * Class definition
 */
typedef struct _MNGmailMailboxClass MNGmailMailboxClass;
struct _MNGmailMailboxClass {
	MNAuthenticatedMailboxClass __parent__;
};


/*
 * Public methods
 */
GType	mn_gmail_mailbox_get_type	(void) G_GNUC_CONST;
#line 386 "src/mn-gmail-mailbox.gob"
char * 	mn_gmail_mailbox_build_name	(const char * username,
					const char * label);
#line 64 "mn-gmail-mailbox.h"

/*
 * Argument wrapping macros
 */
#if defined(__GNUC__) && !defined(__STRICT_ANSI__)
#define MN_GMAIL_MAILBOX_PROP_LOCATION(arg)    	"location", __extension__ ({gchar *z = (arg); z;})
#define MN_GMAIL_MAILBOX_GET_PROP_LOCATION(arg)	"location", __extension__ ({gchar **z = (arg); z;})
#define MN_GMAIL_MAILBOX_PROP_LABEL(arg)    	"label", __extension__ ({gchar *z = (arg); z;})
#define MN_GMAIL_MAILBOX_GET_PROP_LABEL(arg)	"label", __extension__ ({gchar **z = (arg); z;})
#else /* __GNUC__ && !__STRICT_ANSI__ */
#define MN_GMAIL_MAILBOX_PROP_LOCATION(arg)    	"location",(gchar *)(arg)
#define MN_GMAIL_MAILBOX_GET_PROP_LOCATION(arg)	"location",(gchar **)(arg)
#define MN_GMAIL_MAILBOX_PROP_LABEL(arg)    	"label",(gchar *)(arg)
#define MN_GMAIL_MAILBOX_GET_PROP_LABEL(arg)	"label",(gchar **)(arg)
#endif /* __GNUC__ && !__STRICT_ANSI__ */


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
