/* Generated by GOB (v2.0.15)   (do not edit directly) */

#include <glib.h>
#include <glib-object.h>


#include "mn-vfs-message.h"

#ifndef __MN_MAILDIR_MESSAGE_H__
#define __MN_MAILDIR_MESSAGE_H__

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/*
 * Type checking and casting macros
 */
#define MN_TYPE_MAILDIR_MESSAGE	(mn_maildir_message_get_type())
#define MN_MAILDIR_MESSAGE(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), mn_maildir_message_get_type(), MNMaildirMessage)
#define MN_MAILDIR_MESSAGE_CONST(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), mn_maildir_message_get_type(), MNMaildirMessage const)
#define MN_MAILDIR_MESSAGE_CLASS(klass)	G_TYPE_CHECK_CLASS_CAST((klass), mn_maildir_message_get_type(), MNMaildirMessageClass)
#define MN_IS_MAILDIR_MESSAGE(obj)	G_TYPE_CHECK_INSTANCE_TYPE((obj), mn_maildir_message_get_type ())

#define MN_MAILDIR_MESSAGE_GET_CLASS(obj)	G_TYPE_INSTANCE_GET_CLASS((obj), mn_maildir_message_get_type(), MNMaildirMessageClass)

/*
 * Main object structure
 */
#ifndef __TYPEDEF_MN_MAILDIR_MESSAGE__
#define __TYPEDEF_MN_MAILDIR_MESSAGE__
typedef struct _MNMaildirMessage MNMaildirMessage;
#endif
struct _MNMaildirMessage {
	MNVFSMessage __parent__;
};

/*
 * Class definition
 */
typedef struct _MNMaildirMessageClass MNMaildirMessageClass;
struct _MNMaildirMessageClass {
	MNVFSMessageClass __parent__;
};


/*
 * Public methods
 */
GType	mn_maildir_message_get_type	(void) G_GNUC_CONST;

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
