/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_AUTHENTICATED_MAILBOX_PRIVATE_H__
#define __MN_AUTHENTICATED_MAILBOX_PRIVATE_H__

#include "mn-authenticated-mailbox.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#line 25 "src/mn-authenticated-mailbox.gob"

#include <gtk/gtk.h>

#line 18 "mn-authenticated-mailbox-private.h"
struct _MNAuthenticatedMailboxPrivate {
#line 67 "src/mn-authenticated-mailbox.gob"
	gpointer get_password_request;
#line 68 "src/mn-authenticated-mailbox.gob"
	gpointer set_password_request;
#line 75 "src/mn-authenticated-mailbox.gob"
	gboolean checking;
#line 79 "src/mn-authenticated-mailbox.gob"
	GtkWidget * auth_dialog;
#line 84 "src/mn-authenticated-mailbox.gob"
	char * prompted_password_keyring;
#line 85 "src/mn-authenticated-mailbox.gob"
	guint32 prompted_password_item_id;
#line 32 "mn-authenticated-mailbox-private.h"
};
#line 251 "src/mn-authenticated-mailbox.gob"
void 	mn_authenticated_mailbox_parse_username	(MNAuthenticatedMailbox * self, char ** username, char ** domain);
#line 36 "mn-authenticated-mailbox-private.h"
#line 278 "src/mn-authenticated-mailbox.gob"
gboolean 	mn_authenticated_mailbox_fill_password	(MNAuthenticatedMailbox * self, gboolean may_prompt);
#line 39 "mn-authenticated-mailbox-private.h"
#line 341 "src/mn-authenticated-mailbox.gob"
void 	mn_authenticated_mailbox_auth_failed	(MNAuthenticatedMailbox * self);
#line 42 "mn-authenticated-mailbox-private.h"

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
