/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_GMAIL_MAILBOX_PRIVATE_H__
#define __MN_GMAIL_MAILBOX_PRIVATE_H__

#include "mn-gmail-mailbox.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#line 24 "src/mn-gmail-mailbox.gob"

#include <libgnomevfs/gnome-vfs.h>

#line 18 "mn-gmail-mailbox-private.h"
struct _MNGmailMailboxPrivate {
#line 54 "src/mn-gmail-mailbox.gob"
	GnomeVFSURI * uri;
#line 22 "mn-gmail-mailbox-private.h"
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
