/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_SYSTEM_VFS_MAILBOX_PRIVATE_H__
#define __MN_SYSTEM_VFS_MAILBOX_PRIVATE_H__

#include "mn-system-vfs-mailbox.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
