/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_VFS_MAILBOX_BACKEND_PRIVATE_H__
#define __MN_VFS_MAILBOX_BACKEND_PRIVATE_H__

#include "mn-vfs-mailbox-backend.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

struct _MNVFSMailboxBackendPrivate {
#line 47 "src/mn-vfs-mailbox-backend.gob"
	unsigned int queue_check_id;
#line 16 "mn-vfs-mailbox-backend-private.h"
};
#line 70 "src/mn-vfs-mailbox-backend.gob"
void 	mn_vfs_mailbox_backend_monitor	(MNVFSMailboxBackend * self, int check_id, const char * uri, GnomeVFSMonitorType type);
#line 20 "mn-vfs-mailbox-backend-private.h"
#line 112 "src/mn-vfs-mailbox-backend.gob"
void 	mn_vfs_mailbox_backend_queue_check	(MNVFSMailboxBackend * self);
#line 23 "mn-vfs-mailbox-backend-private.h"

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
