/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_EVOLUTION_CLIENT_PRIVATE_H__
#define __MN_EVOLUTION_CLIENT_PRIVATE_H__

#include "mn-evolution-client.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

struct _MNEvolutionClientPrivate {
#line 41 "src/mn-evolution-client.gob"
	gboolean name_owner_signal_connected;
#line 16 "mn-evolution-client-private.h"
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
