/* Generated by GOB (v2.0.15)   (do not edit directly) */

#ifndef __MN_EVOLUTION_MAILBOX_PROPERTIES_PRIVATE_H__
#define __MN_EVOLUTION_MAILBOX_PROPERTIES_PRIVATE_H__

#include "mn-evolution-mailbox-properties.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#line 24 "src/mn-evolution-mailbox-properties.gob"

#include "mn-evolution-client.h"

#line 18 "mn-evolution-mailbox-properties-private.h"
struct _MNEvolutionMailboxPropertiesPrivate {
#line 41 "src/mn-evolution-mailbox-properties.gob"
	GtkWidget * tree;
#line 43 "src/mn-evolution-mailbox-properties.gob"
	char * name;
#line 24 "mn-evolution-mailbox-properties-private.h"
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
